function [XX,KOB,VAR,FEZ,ITS,VTS]=FUPSCAN05(Filename)
%function [XX,KOB,VAR,FEZ,ITS,VTS]=FUPSCAN05(Filename)
%Datenaufnahme aus einem Funktionsplanfile *.mdl oder *.slx, einschlielich Teil-
%funktionsplanerkennung, zur spteren Syntax- und Eingabefehlerprfung.
%Z.Z. werden folgende 16 Blcke der FUP-Blockbibliothek ausgewertet:
%                     ZA, ZX, ZE, ZF, EX, KB, FUV, FVZ
%                     A2V, A3V, A2Z, A3Z, P2V, P3V, P2Z, P3Z
%Von der SIMULINK Standard-Bibliothek werden z.Z. ausgewertet: In, Out, SS
%Einschrnkungen: Kein "Break Library Link" fr FUP-Bibliotheksblcke
%                 Keine Teilsystemnamen mit Namen von FUP-Blcken beginnen
%                 Keine Vernderung von Namen der FUP-Bibliotheksblcke
%                 Keine Blocknamen mit ENTER o.. abschlieen
%                 Keine Verbindung von FUP-Teilsystemen mit Konnektorblcken
%                 Keine Multiplexer, Demultiplexer o.. im Funktionsplan
%Filename: Filenamenstring ohne Endung ".mdl"

%  Copyright (c) 1997-2013. All Rights Reserved. 
%                     Dr. Gert-Helge Geitner; TU Dresden, Fak. EuI,
%                     Elektrotechnisches Institut (ETI); Mommsenstr. 13;
%                     D-01062 Dresden, Germany;
%                     http://eeiwzg.et.tu-dresden.de/ae2_files/ae_7_6e.htm

%XX:       Zustands-, Ereignis- und Verzweigungsblockinformationen
%          Matrixzeilenaufbau=[QZN ZIT Skode SNr Tkode BN FN Offset Z_ZIT1 ZSK1 ..]
%          QZN=Quellzeilennummer
%          ZIT=Zeilennummer in ITS (0=Hauptsystem - ohne Zeile in ITS)
%          Skode=Zahlenkode fr FUP-Block als Namensersatz
%          SNr=Zahlenkode fr Simulink Blocknumerierung
%          Tkode=BlockTypeKode: siehe "if" in BlockScan
%          BN/FN=Block- bzw. Funktionsplannummer im FUP
%          Offset=Offset fr variables Speichern ab Spalte 8 
%                = 2 * Anzahl der Verbindungen zu FUP-Bib. Blcken
%          Z_ZIT...=Ziel-ZIT
%          ZSK...=ZielSimulinkKode nach FUP-Bib. fortlaufend
%KOB:      KB-Blockinformationen
%          Matrixzeilenaufbau=[QZN ZIT Skode SNr Z_ZIT VKB Vkode1 VarName1]
%                             [                   ...      VKode2 VarName2]
%                             Zeilenanzahl pro KB-Block = Variablenanzahl
%          VKB=Verwendung des KB-Blocks an einem Zustandsausgang bzw. in FUP-Bib.
%              ja=SNr des SrcBlocks(auch falscher FUP-Block); nein=0(fremder Block)
%          Vkode...=VariablennamenKode
%          VarName=ASCII-Kode des Variablennamens
%VAR:      FUV-Blockinformationen
%          Matrixzeilenaufbau=[QZN ZIT Skode SNr Vkode (Z_ZIT ZSK) VarName]
%          (ZSK: fr Teilsysteme richtig, sollte sonst nicht auftreten!)
%          VarName=ASCII-Kode des Variablennamens
%FEZ:      FVZ-Blockinformationen
%          Matrixzeilenaufbau=[QZN ZIT Skode SNr Z_ZIT VEX FN BBN1 BBN2 ...]
%          VEX=Verwendung zur EX-Blockspeisung am Enable-Eingang
%              ja=SNr des DstBlocks; 0=nein
%          BBN...=BoolscheGleichung BlockNummer
%ITS:      Information Teilsysteme
%          Matrixzeilenaufbau=[TSE Kode Teilsystemname]
%          TSE=Teilsystemebene 0=Hauptsystem - nicht gespeichert;
%                              1=erste Teilsystemebene, 2=zweite ...)
%          Kode=Zahlenquivalent fr String des Teilsystemnames
%INTERN
%VTS:      Verbindung Teilsysteme
%          Matrixzeilenaufbau=[QZN ZIT S_ZIT TKode1 SNr1 P1 TKode2 SNr2 P2]
%          bzw. falls "Line: FUP-Block ---> TS-Block":
%                            =[ 0  ZIT ZEILE SPALTE  -  SCH TKode2 SNr2 P2]
%          S_ZIT=Schalt-ZIT von / zu bergeordneter Ebene
%          TKode: 0=FUP-Block;
%                 100=Ausgang-SubSystem-Block; 101=Inport-Block; 
%                 102=Outport-Block; 103=Eingang-SubSystem-Block;
%          P1/P2=Src- bzw. Dst-Portnummer fr TKode={100, 101, 102, 103}

global fid T_03 T_05 T_06 T_07 T_08 T_09
global T_14 T_15 T_16 T_17 T_18
global T_00 T_04 T_1a T_1b T_1c T_19 T_20 T_21 T_22 T_27 T_28
global WM_01 WM_02 WT_01 WT_03
ANFUE=34; KOMMA=44; IGROS=73; EKLAU=91; EKLZU=93; SPACE=32; HOTAB=09;

T_00='Analysed DstBlock line does not contain two: "';
T_01='File does not exist!';
T_02='File may not been closed in proper form!';
T_03='Analysed block name line does not contain two: "';
T_04='Analysed SrcBlock line does not contain two: "';
T_05='Analysed block number line does not contain two: "';
T_06='Analysed Function Chart number line does not contain two: "';
T_07='FUV block: source line extension incorrect!';
T_08='FVZ block: source line extension incorrect!';
T_09='KB block: source line extension incorrect!';
T_10a=[char(9) char(9) 'CHECK NOT POSSIBLE!'];
T_10b='"Break Library Link" was set for one or more than one FUP library blocks.';
T_10c=[char(9) char(9) char(9) 'OR'];
T_10d='Subsystem name starts with FUP add-on library block name.';
T_10=[char(10) T_10a char(10) T_10b char(10) T_10c char(10) T_10d];
T_10e='In / Out port name starts with FUP add-on library block name.';
T_10F=[char(10) T_10a char(10) T_10e];
T_1a='Block: SubSystem ZIT or Port error';
T_1b='Block: Inport ZIT error';
T_1c='Block: VTS code error';

T_11='Block {';                     %Blockdefinition
T_12='Line {';                      %Verbindungsdefinition
T_13='SubSystem';                   %Teilsystemdefinition (maskiertes Subsystem)
T_14='BN';                          %Blocknummerdefinition fr Zustand
T_15='FN';                          %Funktionsplannummerdefinition fr Zustand
T_16='Ns';                          %FUV-Variablennamendefinition
T_17='SE';                          %KB-Variablensetzdefinition
T_18='BG';                          %FVZ-Boolsche Gleichung
T_19='}';                           %Definition fr Schlieen
T_20='DstBlock';                    %DstBlockdefinition
T_21='Branch {';                    %Branchdefinition fr Line-Verzweigung
T_22='enable';                      %EX-Definition In2 = enable-Eingang
T_23='Reference';                   %Blocktyp Zusatzbibliothek
T_24='Inport';                      %Eingangsportdefinition (maskiertes Subsystem)
T_25='Outport';                     %Ausgangsportdefinition (maskiertes Subsystem)
T_26='Port';                        %Portnummerdefinition (Inport-, Outportblcke)
T_27='Labels';                      %Labeldefinition unmittelbar nach Verb.-def.
T_28='SrcPort';                     %Schlsselwort SourcePort in Quellfile
T_29='Name';                        %Schlsselwort Name in Quellfile
%
T_30='FromPortIcon';                %alte Def.: on - ohne Hochkomma; NEU dies mit
T_31='SID';                         %NEUERE Definition zus. zwischen Name und Ports

WM_01='FUPSCAN message ';
WM_02='from LineScan';
WT_01=['  --->  Guess: File does not contain any FUP. ' char(10) char(10) ...
     'Otherwise any FUP of this file could feature branch errors. ' ...
      char(10) 'For instance:' char(10) 'Wrong direct signal branch ' ...
      'without use of appropriate blocks in FUP add-on block library!'];
WT_02=['A subsequent line was read, in order to avoid following error message ' ...
       'and a break afterwards: ' char(10) ...
     '"??? Attempted to access x(2); index out of bounds because numel(x)=0." ' ...
       char(10) char(10) ...
       'Guess: a source line exceeding, very long block name.'];
WT_03=['A subsequent line was read, in order to avoid following error message ' ...
       'and a break afterwards: ' char(10) ...
       '"??? Error using ==> eq / Matrix dimensions must agree. ' ...
       char(10) char(10) ...
       'Guess: a source line exceeding, very long block name.'];  

fid=fopen([Filename '.mdl'],'rt');  %ffnen eines *.mdl Files zum Lesen
if fid==-1 
   fid=fopen([Filename '.slx'],'rt');  %ffnen eines *.slx Files zum Lesen
   if fid==-1 error(T_01); end         %fid=-1 wenn nicht existent
end
QZN=0; XX(1,8)=0; XL=0; KOB=[]; KL=0; VAR=[]; VL=0; FEZ=[]; ZL=0;
ITS=[]; TSE=0; TSZ=0; LBE=0; ZIT=0; VTS=[]; VTSZ=0;
S_ZIT=0;                            %Falls keine Teilsysteme definiert sind
while feof(fid)==0
 line=fgetl(fid); QZN=QZN+1;        %Einlesen ohne Zeilenwechselkode
 if findstr(line,T_11)              %auf Block-Definition prfen
    if LBE==1 TSE=TSE-1;            %Teilsystemende erkannt, Ebene erhhen
              LBE=0;                %LineBlockErkennung auf Block schalten
              if TSE==0 ZIT=0;      %Hauptsystem einstellen (keine ITS Zeile)
              else ZIT=ZIT-1;       %bergeordnetes Teilsystem einstellen
                   while ITS(ZIT,1)>TSE ZIT=ZIT-1; end   
                                    %berspringen mehrfach gleicher Ebenen
              end
    end 
    line=fgetl(fid); QZN=QZN+1;     %nchste Zeile einlesen = BlockTyp Definition
    if findstr(line,T_23)           %Reference-Definition erkannt
       line=fgetl(fid); QZN=QZN+1;  %nchste Zeile = BlockNameDefinition einlesen
       if findstr(line,T_29)        %Name? - Nein: nur symb.(leere) -Reference-def.
          x=find(line==ANFUE);         %Zeichenposition von " suchen
          [XX,XL,KOB,KL,VAR,VL,FEZ,ZL,QZN]=...
             BlockScan(XX,XL,KOB,KL,VAR,VL,FEZ,ZL,QZN,line,x,ZIT); end
    elseif findstr(line,T_13)       %SubSystem-Definition erkannt
       line=fgetl(fid); QZN=QZN+1;  %nchste Zeile = Blocknamedefinition einlesen
       x=find(line==ANFUE);         %Zeichenposition von " suchen
       if ~isempty(x)&isempty(findstr(line,T_30))   %on(alt)/"FromPortIcon"(neu)
                                    %Ausschliessen von: Subsystem -->ShowPortLabels
          TSZ=TSZ+1;                   %ITS um eine Zeile erweitern
          TSE=TSE+1;                   %eine Ebene tiefer einstellen
          ITS(TSZ,1)=TSE;              %in neuer Zeile Spalte 1 Ebene speichern
          AW=x(1)+1; j=x(2)-AW-1;      %Zeichenkettenlnge-1 berechnen
          switch j                     %Blocknamenskode berechnen
             case 0,   Kode=(line(AW)-47)*77; K2=0; K3=0;
             case 1,   K2=(line(AW)-47)*77+(line(AW+1)-47)*78; Kode=K2; K3=0;
             otherwise K2=(line(AW)-47)*77+(line(AW+1)-47)*78;
                       K3=K2+(line(AW+2)-47)*79; Kode=K3; K4=80; %gemischte Kodierg
                       for i=3:j K4=K4*80; Kode=Kode+(line(AW+i)-47)*K4; end; end
          switch K2                                  %TS-Namenbeginn ausschlieen:
             case {4715; 6509; 5027; 5105; 4892; 3638} %ZA, ZX, ZE, ZF, EX, KB
             error(T_10); end
          switch K3                                  %TS-Namenbeginn ausschlieen:
             case {7816; 8210; 4701; 4779; 5017; 5095; 5856; 5934; 6172; 6250}
             error(T_10); end    %FUV, FVZ, A2V, A3V, A2Z, A3Z, P2V, P3V, P2Z, P3Z       
          ITS(TSZ,2)=Kode;             %Kode des TS Namens in Spalte 2 speichern
          ITS(TSZ,3:j+3)=line(AW:x(2)-1); %Blockname ab Spalte 3 speichern
          S_ZIT=ZIT;                   %alte Zeilennummer=bergeordnete Ebene
          ZIT=TSZ;                     %neue Zeilennummer in ITS bergeben
          HVek=[QZN ZIT S_ZIT 100 Kode]; %TKode auf TS-Ausgnge einstellen
          line=fgetl(fid); QZN=QZN+1;  %nchste Zeile = Ports Definition einlesen
          if  isempty(strfind(line,[T_26 's'])) %enthlt Zeile nicht "Ports" Def.?
              if  isempty(strfind(line,T_31))   %Zeile mit "SID" NEUE-MDL-Def.?
                  warndlg(WT_02,WM_01); end     %vielleicht Blockname ber 1 Zeile?
              line=fgetl(fid); QZN=QZN+1;       %einlesen: nchste Zeile = "Ports"?
              if  ~isempty(strfind(line,T_31))  %UeberlangerName u. "SID" Def.?
                  line=fgetl(fid); QZN=QZN+1; end; end %FolgeZeile nun = "Ports"?
          x=find(line==KOMMA|line==EKLAU|line==EKLZU);
          hs=line(x(2)+1:x(3)-1);      %ALT: nur diese Zeile ohne "if" vorher
          j=str2num(hs);               %Anzahl der TS-Ausgangsports bestimmen
          for i=1:j                    %Soviele Zeilen wie Ausgnge anlegen
             VTS(VTSZ+i,1:5)=HVek(1,1:5);
             VTS(VTSZ+i,6)=i; end      %Portnummer
          VTSZ=VTSZ+j;                 %VTS um j Zeilen erweitern
          HVek(4)=103;                 %TKode auf TS-Eingnge einstellen
          hs=line(x(1)+1:x(2)-1); 
          j=str2num(hs);               %Anzahl der TS-Eingangsports bestimmen
          for i=1:j                    %Soviele Zeilen wie Eingnge anlegen
             VTS(VTSZ+i,1:5)=HVek(1,1:5);
             VTS(VTSZ+i,6)=i; end      %Portnummer
          VTSZ=VTSZ+j;                 %VTS um j Zeilen erweitern
       end
    elseif ~isempty(findstr(line,T_24))|~isempty(findstr(line,T_25)) 
       if findstr(line,IGROS) TKode=101; %Blocktyp: Inport Definition erkannt
       else TKode=102; end               %Blocktyp: Outport Definition erkannt
       line=fgetl(fid); QZN=QZN+1;  %nchste Zeile (2. Blockdef.) = Name einlesen
       if findstr(line,T_29)        %Name? - Nein: nur Symb.-def.(Port=1/BusObject)
          x=find(line==ANFUE);         %Zeichenposition von " suchen
          AW=x(1)+1; j=x(2)-AW-1;      %Zeichenkettenlnge - wie oben!
          switch j                     %Blocknamenskode berechnen
             case 0,   Kode=(line(AW)-47)*77; K2=0; K3=0;
             case 1,   K2=(line(AW)-47)*77+(line(AW+1)-47)*78; Kode=K2; K3=0;
             otherwise K2=(line(AW)-47)*77+(line(AW+1)-47)*78;
                       K3=K2+(line(AW+2)-47)*79; Kode=K3; K4=80; %gemischte Kodierg
                       for i=3:j K4=K4*80; Kode=Kode+(line(AW+i)-47)*K4; end; end
          switch K2              %In-/Outport-Namenbeginn ausschlieen:
             case {4715; 6509; 5027; 5105; 4892; 3638}
             error(T_10F); end    %ZA, ZX, ZE, ZF, EX, KB
          switch K3              %In-/Outport-Namenbeginn ausschlieen:
             case {7816; 8210; 4701; 4779; 5017; 5095; 5856; 5934; 6172; 6250}
             error(T_10F); end    %FUV, FVZ, A2V, A3V, A2Z, A3Z, P2V, P3V, P2Z, P3Z
          line=fgetl(fid); QZN=QZN+1;  %nchste Zeile = Port >>>sicher!<<< suchen
          while isempty(findstr(line,T_19))& ... % '_Port_' mit 4 Komb. oder '}' ??
                isempty(findstr(line,[SPACE T_26 SPACE]))& ...
                isempty(findstr(line,[SPACE T_26 HOTAB]))& ...
                isempty(findstr(line,[HOTAB T_26 SPACE]))& ...
                isempty(findstr(line,[HOTAB T_26 HOTAB]))
                line=fgetl(fid); QZN=QZN+1; end
            if   findstr(line,T_19) j=1; %while-Ende mit "}" (Blockdef.-ende)
                                         %In-/Outport Nummer 1 wird nicht definiert
            else x=find(line==ANFUE);    %Nummer mit Hochkomma finden
                 if  isempty(x)     %SONDERFALL: erweiterte Portdefinitionen
                     line=fgetl(fid); QZN=QZN+1; %Folgezeile mit Portnummer
                     hs_begin=max(find(line==' ')); hs_end=length(line);
                     hs=line(hs_begin:hs_end); %String ohne Hochkomma finden
                 else
                     hs=line(x(1)+1:x(2)-1); %String fuer Portnummer lesen
                 end
                 j=str2num(hs); end      %Nummer "j" des Ports berechnen
            VTSZ=VTSZ+1;            %VTS um 1 Zeile erweitern
            VTS(VTSZ,1)=QZN;        %Quellzeilennummer
            VTS(VTSZ,2)=ZIT;        %ZIT fr Blcke innerhalb des Teilsystems
                                    %S_ZIT hier nicht sinnvoll
            VTS(VTSZ,4)=TKode;      %TKode
            VTS(VTSZ,5)=Kode;       %SNr
            VTS(VTSZ,6)=j;          %Portnummer
       end
    end
 elseif findstr(line,T_12)          %auf Line-Definition prfen
    if LBE==0 LBE=1; end            %LineBlockErkennung auf Line schalten
    line=fgetl(fid); QZN=QZN+1;     %nchste Zeile einlesen = SrcBlock-Definition
    x=find(line==ANFUE);            %Zeichenposition von " suchen
    [XX,KOB,VAR,FEZ,QZN,VTS,VTSZ]=...
                         LineScan(XX,KOB,VAR,FEZ,QZN,line,x,ZIT,S_ZIT,VTS,VTSZ);
 end
end
status=fclose(fid);                 %Schlieen eines Files
if status==-1 error(T_02); end      %Status=0 i.O.; =-1 Fehler
clear global fid T_03 T_05 T_06 T_07 T_08 T_09
clear global T_14 T_15 T_16 T_17 T_18
clear global T_00 T_04 T_1a T_1b T_1c T_19 T_20 T_21 T_22 T_27 T_28
clear global WM_01 WM_02 WT_01 WT_03
if ~isempty(ITS)
[XX,KOB,VAR,FEZ]=TS_Scan(XX,KOB,VAR,FEZ,VTS); %Teilsysteme auflsen
end

function [XX,XL,KOB,KL,VAR,VL,FEZ,ZL,QZN]=...
                          BlockScan(XX,XL,KOB,KL,VAR,VL,FEZ,ZL,QZN,line,x,ZIT)
global fid T_03 T_05 T_06 T_07 T_08 T_09
global T_14 T_15 T_16 T_17 T_18
SPACE=32; ANFUE=34; KAUND=38; HOCKO=39; NULL=48; NEUN=57;
SEMIK=59; ISTGL=61; SENKS=124;
ZA=4715;  ZX=6509;  ZE=5027;  ZF=5105;  EX=4892;  KB=3638;  FUV=7816; FVZ=8210;
A2V=4701; A3V=4779; A2Z=5017; A3Z=5095; P2V=5856; P3V=5934; P2Z=6172; P3Z=6250;
if length(x)~=2                         %Zeile sollte zwei " enthalten
   Quellzeilennummer=QZN, error(T_03); end
ii=line(x(1)+1:x(2)-1);                 %NamensKodierung
j=x(2)-x(1)-1;                          %Zeichenkettenlnge
K2=(ii(1)-47)*77+(ii(2)-47)*78;         %Kode fr ersten zwei Zeichen
K3=K2+(line(x(1)+3)-47)*79;             %Kode fr ersten drei Zeichen
if K2==ZX|K2==ZE|K2==ZA|K2==ZF
   XL=XL+1;                             %Zustandsblock erkannt
   XX(XL,1)=QZN;                        %Spalte1=Quellzeilennummer
   XX(XL,2)=ZIT;                        %Spalte2=Teilsystemebene (0=Hauptsystem)
   XX(XL,3)=K2;                         %Spalte3=Skode
      K4=80; for i=3:j K4=K4*80; K2=K2+(ii(i)-47)*K4; end %gemischte Kodierung!
   XX(XL,4)=K2;                         %Spalte4=SNr
   XX(XL,5)=10;                         %Spalte5=Tkode=Zustandsblock=10
      line=fgetl(fid); QZN=QZN+1;       %Blocknummer-Definition suchen
      while isempty(findstr(line,T_14)) line=fgetl(fid); QZN=QZN+1; end
      x=find(line==ANFUE);                 %Zeichenposition von " suchen
      if length(x)~=2 Quellzeilennummer=QZN, error(T_05);  end
      y=line(x(1)+1:x(2)-1);            %Zeile sollte zwei " enthalten
   XX(XL,6)=str2num(y);                 %Spalte6=BN
      line=fgetl(fid); QZN=QZN+1;       %Funktionsplannummer-Definition suchen
      while isempty(findstr(line,T_15)) line=fgetl(fid); QZN=QZN+1; end
      x=find(line==ANFUE);                 %Zeichenposition von " suchen
      if length(x)~=2 Quellzeilennummer=QZN, error(T_06);  end
      y=line(x(1)+1:x(2)-1);            %Zeile sollte zwei " enthalten
   XX(XL,7)=str2num(y);                 %Spalte7=FN
elseif K2==EX
       XL=XL+1;                         %Ereignisblock erkannt
   XX(XL,1)=QZN;                        %Spalte1=Quellzeilennummer
   XX(XL,2)=ZIT;                        %Spalte2=Teilsystemebene (0=Hauptsystem)
   XX(XL,3)=K2;                         %Spalte3=Skode
      K4=80; for i=3:j K4=K4*80; K2=K2+(ii(i)-47)*K4; end %gemischte Kodierung!
   XX(XL,4)=K2;                         %Spalte4=SNr
   XX(XL,5)=80;                         %Spalte5=Tkode=Ereignisblock=80
elseif K2==KB                           %Tkode=Kommentarblock=20
       KL=KL+1;                         %Kommentarblock erkannt
   KOB(KL,1)=QZN;                       %Spalte1=Quellzeilennummer
   KOB(KL,2)=ZIT;                       %Spalte2=Teilsystemebene (0=Hauptsystem)
   KOB(KL,3)=K2;                        %Spalte3=Skode
      K4=80; for i=3:j K4=K4*80; K2=K2+(ii(i)-47)*K4; end %gemischte Kodierung!
   KOB(KL,4)=K2;                        %Spalte4=SNr
      line=fgetl(fid); QZN=QZN+1;       %Variablendefinition suchen
      while isempty(findstr(line,T_17)) line=fgetl(fid); QZN=QZN+1; end
      x=find(line==HOCKO);              %Zeichenposition von ' suchen
      while length(x)~=2                %Quellzeilenverlngerung?
         ii=fgetl(fid); QZN=QZN+1;      %Folgezeile einlesen
         if ii(1)~=ANFUE                %Korrekte Verlngerung?
            Quellzeilennummer=QZN, error(T_09); end
         j=length(line); jj=length(ii); line(j:j+jj-2)=ii(2:jj);
         x=find(line==HOCKO);           %Verlngerung mit Ausschlu von ANFUE
      end                               %am Ende von line bzw. Anfang von ii
      x=find(line==HOCKO|line==SEMIK|line==ISTGL); %Zeichenposition von '=; suchen
      j=(length(x)-2)/2;                %Anzahl der Variablen bestimmen
      for i=1:j K2=0; ii=1; K4=1;       %AW's
          if i>1 KL=KL+1;
   KOB(KL,1:6)=KOB(KL-1,1:6); end       %j Zeilen pro KB-Block
          for jj=x(2*i-1)+1:x(2*i)-1 y=line(jj); %Leerzeichen ausschlieen
              if y~=SPACE K2=K2+(y-47)*K4; K4=K4*80; %Komplett neue Kodierung!
   KOB(KL,7+ii)=y;                      %VarName ASCII-Kode ab Spalte 8 
                 ii=ii+1; end, end
   KOB(KL,7)=K2; end                    %Spalte7=Vkode laden
elseif K3==A2V|K3==A3V|K3==A2Z|K3==A3Z
       XL=XL+1;                         %Alternativblock erkannt
   XX(XL,1)=QZN;                        %Spalte1=Quellzeilennummer
   XX(XL,2)=ZIT;                        %Spalte2=Teilsystemebene (0=Hauptsystem)
   XX(XL,3)=K3;                         %Spalte3=Skode
      K4=80; for i=4:j K4=K4*80; K3=K3+(ii(i)-47)*K4; end %gemischte Kodierung!
   XX(XL,4)=K3;                         %Spalte4=SNr
   XX(XL,5)=40;                         %Spalte5=Tkode=Alternativblock=40
elseif K3==P2V|K3==P3V|K3==P2Z|K3==P3Z
       XL=XL+1;                         %Parallelblock erkannt
   XX(XL,1)=QZN;                        %Spalte1=Quellzeilennummer
   XX(XL,2)=ZIT;                        %Spalte2=Teilsystemebene (0=Hauptsystem)
   XX(XL,3)=K3;                         %Spalte3=Skode
      K4=80; for i=4:j K4=K4*80; K3=K3+(ii(i)-47)*K4; end %gemischte Kodierung!
   XX(XL,4)=K3;                         %Spalte4=SNr
   XX(XL,5)=50;                         %Spalte5=Tkode=Parallelblock=50
elseif K3==FUV                          %Tkode=FUP_Variablenblock=25
       VL=VL+1;                         %FUP_Variablenblock erkannt
   VAR(VL,1)=QZN;                       %Spalte1=Quellzeilennummer
   VAR(VL,2)=ZIT;                       %Spalte2=Teilsystemebene (0=Hauptsystem)
   VAR(VL,3)=K3;                        %Spalte3=Skode
      K4=80; for i=4:j K4=K4*80; K3=K3+(ii(i)-47)*K4; end %gemischte Kodierung!
   VAR(VL,4)=K3;                        %Spalte4=SNr
      line=fgetl(fid); QZN=QZN+1;       %Variablennamen-Definition suchen
      while isempty(findstr(line,T_16)) line=fgetl(fid); QZN=QZN+1; end
      x=find(line==HOCKO);              %Zeichenposition von ' suchen
      while length(x)~=2                %Quellzeilenverlngerung?
         ii=fgetl(fid); QZN=QZN+1;      %Folgezeile einlesen
         if ii(1)~=ANFUE                %Korrekte Verlngerung?
            Quellzeilennummer=QZN, error(T_07); end
         j=length(line); jj=length(ii); line(j:j+jj-2)=ii(2:jj);
         x=find(line==HOCKO);           %Verlngerung mit Ausschlu von ANFUE
      end                               %am Ende von line bzw. Anfang von ii
      K3=0; ii=1; K4=1;                 %AW's
      for i=x(1)+1:x(2)-1 y=line(i);       %Leerzeichen und ' ausblenden
          if y~=SPACE K3=K3+(y-47)*K4; K4=K4*80; %Komplett neue Kodierung
   VAR(VL,7+ii)=y;                      %VarName ASCII-Kode ab Spalte 8
                      ii=ii+1; end, end %denn: Spalte 6/7: (Z_ZIT/ZSK ?!)
   VAR(VL,5)=K3;                        %Spalte5=Vkode
elseif K3==FVZ                          %Tkode=BlockFehlervorzustand=30
       ZL=ZL+1;                         %Block Fehlervorzustand erkannt
   FEZ(ZL,1)=QZN;                       %Spalte1=Quellzeilennummer
   FEZ(ZL,2)=ZIT;                       %Spalte2=Teilsystemebene (0=Hauptsystem)
   FEZ(ZL,3)=K3;                        %Spalte3=Skode
      K4=80; for i=4:j K4=K4*80; K3=K3+(ii(i)-47)*K4; end %gemischte Kodierung!
   FEZ(ZL,4)=K3;                        %Spalte4=SNr
      line=fgetl(fid); QZN=QZN+1;       %Funktionsplannummer Definition suchen
      while isempty(findstr(line,T_15)) line=fgetl(fid); QZN=QZN+1; end
      x=find(line==ANFUE);              %Zeichenposition von " suchen
      if length(x)~=2 Quellzeilennummer=QZN, error(T_06); end
      y=line(x(1)+1:x(2)-1);            %Zeile sollte zwei " enthalten
   FEZ(ZL,7)=str2num(y);                %Spalte7=FN
      line=fgetl(fid); QZN=QZN+1;       %Definition Boolsche Gleichung suchen
      while isempty(findstr(line,T_18)) line=fgetl(fid); QZN=QZN+1; end
      x=find(line==HOCKO);              %Zeichenposition von ' suchen
      while length(x)~=2                %Quellzeilenverlngerung?
         ii=fgetl(fid); QZN=QZN+1;      %Folgezeile einlesen
         if ii(1)~=ANFUE                %Korrekte Verlngerung?
            Quellzeilennummer=QZN, error(T_08); end
         j=length(line); jj=length(ii); line(j:j+jj-2)=ii(2:jj);
         x=find(line==HOCKO);           %Verlngerung mit Ausschlu von ANFUE
      end                               %am Ende von line bzw. Anfang von ii
      x=find(line==HOCKO|line==SENKS|line==KAUND); %Zeichenpositionen suchen
      j=length(x)-1;                    %Anzahl verknpfter Blcke
      for i=1:j K3='0'; ii=1;           %AW's
          for jj=x(i)+1:x(i+1)-1 y=line(jj); %Nur Zahlen ausfiltern
              if y>=NULL&y<=NEUN K3(ii)=y; ii=ii+1; end; end
   FEZ(ZL,7+i)=str2num(K3); end         %Spalte8_ff=BNN1 bis BNN... laden
end

function [XX,KOB,VAR,FEZ,QZN,VTS,VTSZ]=...
                           LineScan(XX,KOB,VAR,FEZ,QZN,line,x,ZIT,S_ZIT,VTS,VTSZ)
global fid T_00 T_04 T_1a T_1b T_1c T_19 T_20 T_21 T_22 T_27 T_28
global WM_01 WM_02 WT_01 WT_03
ANFUE=34; SPACE=32; HOTAB=09;
ZA=4715;  ZX=6509;  ZE=5027;  ZF=5105;  EX=4892;  KB=3638;  FUV=7816; FVZ=8210;
A2V=4701; A3V=4779; A2Z=5017; A3Z=5095; P2V=5856; P3V=5934; P2Z=6172; P3Z=6250;
if length(x)~=2                         %Zeile sollte zwei " enthalten
   if ~findstr(line,T_27)               %'Labeldefinition' nach Linedef.-beginn?
      Quellzeilennummer=QZN, error(T_04);
   else
      line=fgetl(fid); QZN=QZN+1; 
      x=find(line==ANFUE);              %Erneute Prfung auf zwei " 
      if length(x)~=2
         Quellzeilennummer=QZN, error(T_04); end; end; end
ii=line(x(1)+1:x(2)-1);                 %NamensKodierung
j=x(2)-x(1)-1;                          %Zeichenkettenlnge
if j>1
   K2=(ii(1)-47)*77+(ii(2)-47)*78;      %Kode fr ersten zwei Zeichen
   K3=K2+(line(x(1)+3)-47)*79;          %Kode fr ersten drei Zeichen
else
   K2=(ii(1)-47)*77; K3=0; end          %Trick fr K3 nicht anwendbar (line<-->ii)  
SCH=0;                                  %entspricht kein Block aus FUP-BiBliothek
SPEI=0;                                 %K2-Speicher fr KB-Dst
if K2==EX|K2==ZX|K2==ZE|K2==ZA|K2==ZF
   SCH=1; K4=80;                        %Auswahl von XX - gemischte Kodierung!
   for i=3:j K4=K4*80; K2=K2+(ii(i)-47)*K4; end %Berechnung von SNr
   GLSB=find(XX(:,4)==K2);              %Zeilen mit diesem SrcBlock Namen in XX
   ZEILE=GLSB(find(XX(GLSB,2)==ZIT));   %davon Zeile mit aktuellem ZIT in XX
   SPALTE=XX(ZEILE,8);                  %Spaltenoffset bez. Spalte 9 in XX
   SPEI=K2;                             %Speicherung von Scr fr KB-Dst
elseif K3==A2V|K3==P2V|K3==A2Z|K3==P2Z|K3==A3V|K3==P3V|K3==A3Z|K3==P3Z
   SCH=1; K4=80;                        %Auswahl von XX - gemischte Kodierung!
   for i=4:j K4=K4*80; K3=K3+(ii(i)-47)*K4; end %Berechnung von SNr
   GLSB=find(XX(:,4)==K3);              %Zeilen mit diesem SrcBlock Namen in XX
   ZEILE=GLSB(find(XX(GLSB,2)==ZIT));   %davon Zeile mit aktuellem ZIT in XX
   SPALTE=XX(ZEILE,8);                  %Spaltenoffset bez. Spalte 9 in XX
elseif K3==FUV
   SCH=2; K4=80;                        %Auswahl von VAR - gemischte Kodierung!
   for i=4:j K4=K4*80; K3=K3+(ii(i)-47)*K4; end %Berechnung von SNr
   SPALTE=6;                            %Spalte von S_ZIT in VAR
   GLSB=find(VAR(:,4)==K3);             %Zeilen mit diesem SrcBlock Namen in VAR
   ZEILE=GLSB(find(VAR(GLSB,2)==ZIT));  %davon Zeile mit aktuellem ZIT in VAR
elseif K3==FVZ
   SCH=3; K4=80;                        %Auswahl von FEZ - gemischte Kodierung!
   for i=4:j K4=K4*80; K3=K3+(ii(i)-47)*K4; end %Berechnung von SNr
   SPALTE=5;                            %Spalte von VEX in FEZ
   GLSB=find(FEZ(:,4)==K3);             %Zeilen mit diesem SrcBlock Namen in FEZ
   ZEILE=GLSB(find(FEZ(GLSB,2)==ZIT));  %davon Zeile mit aktuellem ZIT in FEZ
else                                    %KB entfllt, da Senkenblock
   if j<3 K3=K2;                        %Blocknamenskode allgemein berechnen:
   else K4=80; for i=4:j K4=K4*80; K3=K3+(ii(i)-47)*K4; end; end %gemischte Kodrg!
   if ~isempty(VTS)
      x=find(VTS(:,5)==K3);                    %VTS-Zeilen dieser SNr suchen
      if ~isempty(x)
         i=find(VTS(x(:),3)==ZIT|VTS(x(:),2)==ZIT);%ZIT(LineScan)==S_ZIT(VTS)!
         SCH=VTS(x(i(1)),4);                   %TKode1; bei TS zwei Varianten
         switch SCH
            case {100,103}                           %TKode1=Teilsystem
                      line=fgetl(fid); QZN=QZN+1;    %SrcPort einstellen
                      if  isempty(strfind(line,T_28))  %enthlt Zeile ScrPort Def.?
                          warndlg(WT_03,[WM_01 WM_02]);   %Blockname > eine Zeile!
                          line=fgetl(fid); QZN=QZN+1; end %Folgezeile = ScrPort Def
                      ii=find(line==SPACE|line==HOTAB);
                      Port=str2num(line(max(ii)+1:end));%Portnummer berechnen
                      SCH=100;                       %TKode1: nur TS-Ausgnge!
                      ii=find(VTS(x(i(:)),6)==Port&VTS(x(i(:)),4)==SCH);   
                      ZEILE=x(i(ii(:)));
                      if length(ZEILE)~=1 error(['TS-Src' T_1a]); end
            case 101, ZEILE=x(i(:));                 %TKode1=Inport-Blcke
                      if length(ZEILE)~=1  %ALT nur: error(['TS-Src' T_1b]);
                                           %ABER: ohne FUP --> Verzweigg mglich
                         ZEILE=ZEILE(1);   %Einfach 1. Wert benutzen --> weiter
                         warndlg(['TS-Src' T_1b WT_01],[WM_01 WM_02]);
                      end
            otherwise error(['TS-Src' T_1c]);
         end                            %Outport+TS-Ein entfallen, da Senkenblcke
      end
   end
end                                     %Line Src-Erkennung beendet
if SCH==0 return; end                   %Kein ScrBlock aus FUP-Bibliothek oder VTS
line=fgetl(fid); QZN=QZN+1; SCH_SP=SCH; BUK_D=0;
LWS=1; EES=0;                           %LinescanWhileSchalter; ExEnableSchalter
while LWS~=0
   if findstr(line,T_20)                %DstBlock suchen
      SCH=SCH_SP;                       %ntig falls: 1. DstBlock = nicht FUP/VTS
      BUK_D=0;                          %"Bransch }"-Doppel rcksetzen
      x=find(line==ANFUE);              %Zeichenposition von " suchen
      if length(x)~=2                   %Zeile sollte zwei " enthalten
         Quellzeilennummer=QZN, error(T_00); end
      ii=line(x(1)+1:x(2)-1);                 %NamensKodierung
      j=x(2)-x(1)-1;                          %Zeichenkettenlnge
      if j>1
         K2=(ii(1)-47)*77+(ii(2)-47)*78;      %Kode fr ersten zwei Zeichen
         K3=K2+(line(x(1)+3)-47)*79;          %Kode fr ersten drei Zeichen  
      else K2=(ii(1)-47)*77; K3=0; end        %Trick fr K3 hier unmglich
      K2BT=0; TKode2=0;                       %K2 Blocktyp und TKode2 rcksetzen
      if K2==EX|K2==KB|K2==ZX|K2==ZE|K2==ZA|K2==ZF
         K2BT=K2; K4=80;                %Erkenng von EX- und KB-Blcken / gemischt
         for i=3:j K4=K4*80; K2=K2+(ii(i)-47)*K4; end  %Ber. SNr fr 2-stell. Kode
      elseif K3==A2V|K3==P2V|K3==A2Z|K3==P2Z|K3==A3V|K3==P3V|K3==A3Z|K3==P3Z
         K2=K3; K4=80;                  %SNr in einem Speicher ablegen! / gemischt
         for i=4:j K4=K4*80; K2=K2+(ii(i)-47)*K4; end  %Ber. SNr fr 3-stell. Kode
                                              %FUV,FVZ: entfallen da Quellblcke
      else                                    %SNr. allgemein berechnen: VTS-Block?
         if j>2 K2=K3; K4=80;                 %gemischte Kodierung!
                for i=4:j K4=K4*80; K2=K2+(ii(i)-47)*K4; end; end%Vollst. Nam.-ber.
         if ~isempty(VTS)
            x=find(VTS(:,5)==K2);             %VTS-Zeilen dieser SNr suchen
            if ~isempty(x)
               i=find(VTS(x(:),3)==ZIT|VTS(x(:),2)==ZIT);%ZIT(LineScan)==S_ZIT(VTS)
               TKode2=VTS(x(i(1)),4);         %TKode2; bei TS zwei Varianten
               switch TKode2
                  case {103, 100}             %TKode2=Teilsystem
                     line=fgetl(fid); QZN=QZN+1; %DstPort einstellen
                     EES=1;                   %while-Steuerung von K2BT==EX genutzt
                     ii=find(line==SPACE|line==HOTAB);
                     Port2=str2num(line(max(ii)+1:end)); %Portnummer berechnen
                     if isempty(Port2)        %Portbezeichnung keine Zahl?
                        SCH=0; TKode2=0;      %Abbruch (TS-In=trigger,enable...)
                     else
                        TKode2=103;           %TKode2: nur TS-Eingnge!
                        ii=find(VTS(x(i(:)),6)==Port2&VTS(x(i(:)),4)==TKode2);
                        ZEIL3=x(i(ii(:)));    %Kontrolle, ob genau 1x vorhanden
                        if length(ZEIL3)~=1 error(['TS-Dst' T_1a]); end; end
                  case 102
                     Port2=1;                 %knnen nur Nummer 1 haben
                     ZEIL3=x(i(:));           %Kontrolle, ob genau 1x vorhanden
                     if length(ZEIL3)~=1 error(['TS-Dst' T_1b]); end
                  otherwise
                     error(['TS-Dst' T_1c]); 
               end                      %Inport+TS-Aus entfallen, da Senkenblcke
            else SCH=0; end             %DstBlock ist kein VTS- oder FUP-Bib-Block!
         else SCH=0;                    %DstBlock ist kein VTS- oder FUP-Bib-Block!
         end
      end
      j=SCH+TKode2;                     %SCH={0,1,2,3,100,101}, TKode2={0,102,103}
      if j<100&SCH~=0                   %Fall1 Line: FUP-Block ---> FUP-Block
         if K2BT==KB                       %Dst=KB
            GLSB=find(KOB(:,4)==K2);       %Zeilen mit diesem DstBlock Namen in KOB
            ZEIL2=GLSB(find(KOB(GLSB,2)==ZIT)); %davon Zeile mit akt. ZIT in KOB
            KOB(ZEIL2,5)=ZIT;              %Ziel-ZIT; wegen Teilsystemen ntig
            KOB(ZEIL2,6)=SPEI;             %Src-SNr speichern (Zustands- u. Exbl.)
         elseif SCH==1                     %Src in XX
            XX(ZEILE,9+SPALTE)=ZIT;        %Ziel-ZIT; wegen Teilsystemen ntig
            XX(ZEILE,10+SPALTE)=K2;        %Dst-SNr ber Offset speichern
            SPALTE=SPALTE+2;               %Offset erhhen
            XX(ZEILE,8)=SPALTE;            %Offset speichern
         elseif SCH==3                     %Src=FVZ
            if K2BT==EX                    %Dst=EX???
               line=fgetl(fid); QZN=QZN+1;
               EES=1;                      %Kein Vorschub im Sonderfall
               if findstr(line,T_22)             %Eingang=enable???
                  FEZ(ZEILE,SPALTE)=ZIT;         %Ziel-ZIT
                  FEZ(ZEILE,SPALTE+1)=K2; end, end %FVZ an EX enable Port
         elseif SCH==2                           %Src=FUV
            VAR(ZEILE,SPALTE)=ZIT;               %Ziel-ZIT
            VAR(ZEILE,SPALTE+1)=K2;              %darf hier nicht vorkommen!
         end
      elseif j>200                      %Fall2 Line: TS-Block ---> TS-Block
         VTS(ZEILE,7)=TKode2;
         VTS(ZEILE,8)=K2;
         VTS(ZEILE,9)=Port2;
      elseif j==100|j==101              %Fall3 Line: TS-Block ---> FUP-Block
         VTS(ZEILE,7)=K2BT;             %SCH={2,3} nicht mglich
         VTS(ZEILE,8)=K2;
         if K2BT==EX                    %Dst=EX???
               line=fgetl(fid); QZN=QZN+1;
               EES=1;                   %Kein Vorschub im Sonderfall
               if findstr(line,T_22)    %Eingang=enable Port???
                  VTS(ZEILE,9)=99; end, end   %Kode fr "enable Port"
      elseif SCH~=0                     %Fall4 Line: FUP-Block ---> TS-Block
         VTSZ=VTSZ+1;                   %VTS erweitern
         VTS(VTSZ,1)=0;                 %statt QZN: Kode fr Erweiterung
         VTS(VTSZ,2)=ZIT;               %Src-ZIT
         VTS(VTSZ,3)=ZEILE;             %statt S_ZIT: Zeile in {XX,VAR}
         VTS(VTSZ,4)=SPALTE;            %statt TKode1: Spalte in {XX,VAR}
         VTS(VTSZ,5)=SPEI;              %Src-SNr fr KOB
         VTS(VTSZ,6)=SCH;               %statt P1: SCH fr FUP-Src
         VTS(VTSZ,7)=TKode2;
         VTS(VTSZ,8)=K2;
         VTS(VTSZ,9)=Port2;  
      end
      LWS=LWS-1;                        %unverzweigt: ENDE; verzweigt: weiter
   elseif findstr(line,T_21)            %Definition fr "Branch {" suchen
      if BUK_D==0 LWS=LWS+2; else LWS=LWS+1; end
      BUK_D=1;                          %"Bransch }"-Doppel vorbereiten
   end
   if EES==0 line=fgetl(fid); QZN=QZN+1;
   else EES=0; end                      %Sonderfall FVZ-->EX(enable):kein Vorschub!
   if findstr(line,T_19)                %Definition fr Schlieen suchen
      BUK_D=0;                          %"Bransch }"-Doppel rcksetzen
      LWS=LWS-1;                        %Verzweigt: doppelte Definition --> ENDE
   end
end
