function varargout = FupCheck(varargin)
% FupCheck Application M-file for FupCheck.fig
%    FIG = FupCheck launch FupCheck GUI.
%    FupCheck('callback_name', ...) invoke the named callback.

%  Copyright (c) 1997-2013. All Rights Reserved. 
%                     Dr. Gert-Helge Geitner; TU Dresden, Fak. EuI,
%                     Elektrotechnisches Institut (ETI); Mommsenstr. 13;
%                     D-01062 Dresden, Germany;
%                     http://eeiwzg.et.tu-dresden.de/ae2_files/ae_7_6e.htm

if nargin <= 1                    %LAUNCH GUI mit oder ohne Direktoryangabe
  if nargin == 0 
	 initial_dir = pwd;           %Arbeitsdirektory bleibt nach GUI Ende erhalten.
  elseif nargin == 1 & exist(varargin{1},'dir')  
	 initial_dir = varargin{1};   %Wechsel in diese Direktory - auch nach GUI Ende!
  else
    errordlg('Not a valid directory!','Error input argument:')  
    %errordlg('Keine gltige Direktory vorgegeben!','Fehler Eingangsargument:')
	return
  end
  fig = openfig(mfilename,'reuse');  %ffnen des FIG-Files
  set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));%Systemfarben verwenden
  handles = guihandles(fig);  %Generierung der "handles"-Struktur fr die Callbacks
  if ~exist('handles.dir_spei')      %Ausgangsdirektory speichern
     handles.dir_spei=initial_dir;   %
  end                                %
  handles.FUP=[0 1 1 1 0]; %Initialisierng[FN PrfArt Textausgabe Sackgasse Syntax]
  handles.STEU=[1 1 1 1 1 1 1];      %Initialisierung aller Parameter-Prfschritte
  guidata(fig, handles);             %speichern der "handles"-Struktur
  load_listbox1(initial_dir,handles) %Initialisierung von listbox1
  if nargout > 0                     %Erstes Ausgangsargument = figure handle
    varargout{1} = fig;
  end
elseif ischar(varargin{1})           %INVOKE NAMED SUBFUNCTION OR CALLBACK
  try
    [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
  catch
    disp(lasterr);
  end
end

% ------------------------------------------------------------
% Callback for "Ende" button (Close)
% ------------------------------------------------------------
function varargout = pushbutton1_Callback(h, eventdata, handles, varargin)
pos_size = get(handles.figure1,'Position');
user_response = FC_Ende([pos_size(1)+pos_size(3)/5 pos_size(2)+pos_size(4)/5]);
switch user_response
case {'No','Abort'}        % take no action
case 'Yes'
    cd (handles.dir_spei);     %Ausgangsdirektory wieder einstellen
	delete(handles.figure1)    % Prepare to close GUI application window
end

% --------------------------------------------------------------------
% Callback for "Hilfe" button (Help)
% --------------------------------------------------------------------
function varargout = pushbutton3_Callback(h, eventdata, handles, varargin)
pfad=which('ae_7_6.htm');
if   isempty(pfad)
     warndlg('File "ae_7_6e.htm" was not found.','Info:')
     %warndlg('Das File ae_7_6.htm wurde nicht gefunden.','Information:')
else web(pfad);
end
% --------------------------------------------------------------------
% Callback for "Start" button
% --------------------------------------------------------------------
function varargout = pushbutton2_Callback(h, eventdata, handles, varargin)
global FupCheck_HDL_FUP_
x1=handles.FUP; x2=handles.FileName; x3=handles. STEU;
switch handles.FUP(2)
  case 0
   FC_VOLL(handles.xx,handles.kob,handles.var,handles.fez,handles.its,x1,x2,x3);
  case 1
    if isempty(FupCheck_HDL_FUP_)     %FC_PARA --> Neu- oder Wiederstart
       FupCheck_HDL_FUP_=[1 0 1 1 1 1 1 1 1];%Globaler Merker
                                      %x1=[FN PrfArt Textausgabe Sackgasse Syntax]
    else                              %FC_SYNT ist aktiv
       x3=FupCheck_HDL_FUP_(3:9);     %Einstellung aller ParameterPrfschritte
    end
    FC_PARA(handles.xx,handles.kob,handles.var,handles.fez,handles.its,x1,x2,x3);
  case 2
    if isempty(FupCheck_HDL_FUP_)       %FC_SYNT --> Neu- oder Wiederstart
       FupCheck_HDL_FUP_=[1 0];         %Globaler Merker
       x1(5)=0;                         %[FN PrfArt Textausgabe Sackgasse Syntax]
    else                                %FC_SYNT ist aktiv
       x1(5)=FupCheck_HDL_FUP_(2);      %[Sackgasse Syntax]
    end
    FC_SYNT(handles.xx,handles.kob,handles.var,handles.fez,handles.its,x1,x2);
  case 3
    if isempty(FupCheck_HDL_FUP_)       %FC_SACK --> Neu- oder Wiederstart
       FupCheck_HDL_FUP_=[1 0];         %Globaler Merker
       x1(4)=1;                         %[FN PrfArt Textausgabe Sackgasse Syntax]
    else                                %FC_SACK ist aktiv
       x1(4)=FupCheck_HDL_FUP_(1);      %[Sackgasse Syntax]
    end
    FC_SACK(handles.xx,handles.kob,handles.var,handles.fez,handles.its,x1,x2);
  case 4
    FC_WEIT(handles.xx,handles.kob,handles.var,handles.fez,handles.its,x1,x2);
  otherwise
    errordlg('Please report error code >>>SB01<<<.','Fatal error:','modal');
    %errordlg('Bitte Fehlerkode >>>SB01<<< rckmelden.','Fataler Fehler:','modal');
end
% ------------------------------------------------------------
% Callback for the GUI figure CloseRequestFcn property
% ------------------------------------------------------------
function varargout = figure1_CloseRequestFcn(h, eventdata, handles, varargin)
pushbutton1_Callback(h,eventdata,handles)

% ------------------------------------------------------------
% Callback for listbox1 - Direktory Anzeige
% ------------------------------------------------------------
function varargout = listbox1_Callback(h, eventdata, handles, varargin)
T_00='Error:  file selection';
%T_00='Fehler:  File-Auswahl';
T_00b='Please choose any *.mdl file!';
T_01='Please choose any *.slx or *.mdl file!';
%T_01='Bitte whlen Sie ein *.slx oder *.mdl File aus!';
T_01b='No support for slx model coding in this version';
T_02='There is no FUP in this simulink file!';
%T_02='Dieses SIMULINK-File enthlt keinen Funktionsplan!';
set(handles.pushbutton2,'Enable','off')      %Start sperren
index_selected = get(handles.listbox1,'Value');
file_list = get(handles.listbox1,'String');
if ~isa(file_list,'cell')&file_list=='<empty>' return; end
filename = file_list{index_selected};
if strcmp(get(handles.figure1,'SelectionType'),'open')    %Doppelklick ?
	if  handles.is_dir(handles.sorted_index(index_selected)) %'..'?
		cd (filename)                        %Direktorywechsel in MATLAB!!!
		load_listbox1(pwd,handles)
	else
	   %[path,name,ext,ver] = fileparts(filename);
	   [path,name,ext] = fileparts(filename); %Funktionsdefinition gendert
       switch ext
	   case '.fig'
		   guide (filename)
	   otherwise 
		try
	    	open(filename)
		catch
			errordlg(lasterr,'File Type Error','modal')
		end
	   end	
   end
else                                                     %Einfachklick ?
   if  handles.is_dir(handles.sorted_index(index_selected)) %'..' oder '.'
       set(handles.FupListe,'String','<empty>','Value',[]); % oder 'dir' ?
       return
   else
       %[path,name,ext,ver] = fileparts(filename);
       [path,name,ext] = fileparts(filename); %Funktionsdefinition gendert
       switch ext
       case {'.mdl', '.slx'} %Extension change for Simulink, R2012b
          switch ext
              case {'.slx'}
                  set(handles.FupListe,'String','<empty>','Value',[]);
                  errordlg(T_01b,T_00b,'modal');
                  % copy+ext_zip --> unzip--> cd_simulink_dir --> blockdiagram.xml:
                  %FUP_SLX_PREPARE
                  % slx(utf-8)/xml-File auswerten:
                  %[XX,KOB,VAR,FEZ,ITS]=FUPANALYSE01(blockdiagram); 
                  % cd.. --> cd.. --> del_zip-file --> rmdir(slx) mit subdir
                  %FUP_XML_DELETE
              otherwise
                  [XX,KOB,VAR,FEZ,ITS]=FUPSCAN05(name); %mdl-File auswerten
          %%% hier "end" wenn mdl und slx ausgewerten werden
          AnzFN=setdiff(XX(:,7),0)'; %XX_Spalte_7=FN; jede FN nur einmal laden und 
                                     %FN=0 von Nichtzustandsblcken ausschlieen
          if isempty(AnzFN)
             set(handles.FupListe,'String','<empty>','Value',[]);
             errordlg(T_02,T_00,'modal');
          else
             ScanFN=[];
             for i=1:length(AnzFN)            %Anzahl der FUP's
                 ScanStr=[' FUP-Nr.:  ',num2str(AnzFN(i))];
                 ScanFN=[ScanFN;ScanStr];
             end
             set(handles.FupListe,'String',ScanFN,'Value',[]); %nicht markiert!!
             handles.xx=XX;
             handles.kob=KOB;
             handles.var=VAR;
             handles.fez=FEZ;
             handles.its=ITS;
             handles.FileName=filename;        %fr Prffenster
             guidata(handles.figure1, handles);%fr Matrizen ntig / listbox nicht
          end
          end %%% hier kein "end" (siehe oben), wenn mdl und slx ausgewertet
       otherwise
          set(handles.FupListe,'String','<empty>','Value',[]);
          errordlg(T_01,T_00,'modal');
       end
   end
end
% ------------------------------------------------------------
% Read the current directory and sort the names
% ------------------------------------------------------------
function load_listbox1(dir_path,handles)
T_00='Error:  directory selection';
%T_00='Fehler:  Direktory-Auswahl';
T_01='Please input any valid directory name!';
%T_01='Bitte geben Sie einen gltigen Verzeichnisnamen ein!';
PM=dir(dir_path);                     %Ist Eingabe gltiger Pfad?
px=length(PM);
if px==0|(px==1&PM.isdir==0)          %Kein DirName oder ist FileName
   set(handles.listbox1,'String','<empty>','Value',1); %FileFenster
   errordlg(T_01,T_00,'modal');
else
   cd (dir_path)
   dir_struct = dir(dir_path);
   [sorted_names,sorted_index] = sortrows({dir_struct.name}');
   handles.file_names = sorted_names;
   handles.is_dir = [dir_struct.isdir];
   handles.sorted_index = [sorted_index];
   guidata(handles.figure1,handles)
   set(handles.listbox1,'String',handles.file_names,'Value',1)
   set(handles.edit1,'String',pwd)
end
% ------------------------------------------------------------
% Callback for edit1 - Direktory Anzeige
% ------------------------------------------------------------
function varargout = edit1_Callback(h, eventdata, handles, varargin)
Eingabe=get(handles.edit1,'String');
load_listbox1(Eingabe,handles)
set(handles.pushbutton2,'Enable','off')              %Start sperren
set(handles.FupListe,'String','<empty>','Value',[]); %Keine FUP-Nr.!
% ------------------------------------------------------------
% Callback for listbox2 - Funktionsplannummern
% ------------------------------------------------------------
function varargout = listbox2_Callback(h, eventdata, handles, varargin)
T_00='Error: file selection';
%T_00='Fehler:  File-Auswahl';
T_01='Please choose any file containing a FUP!';
%T_01='Bitte whlen Sie ein File mit einem Funktionsplan aus!';
T_02='Version mismatch: 1 <= FN <= 9 necessary!';
%T_02='Versionsfehler: 1 <= FN <= 9 notwendig!';
ind=get(handles.FupListe,'Value');
str=get(handles.FupListe,'String');
if str(1,1)=='<'
   errordlg(T_01,T_00,'modal');  %<empty> ausgewhlt!
else
   FN=str2num(str(ind,12)); %StringMatrix-Auswahl einlesen + FN berechnen
   if 1<=FN&FN<=9
      set(handles.pushbutton2,'Enable','on')
      handles.FUP(1)=FN;
      guidata(handles.figure1, handles);     %speichern
   else
      errordlg(T_02,T_00,'modal'); %FUP-Bib erweitert / Tools noch nicht
   end
end
% --------------------------------------------------------------------
% Callback for radiobutton1 - vollstndige Prfung (Complete check)
% --------------------------------------------------------------------
function varargout = radiobutton1_Callback(h, eventdata, handles, varargin)
on=[handles.radiobutton1,handles.radiobutton5,handles.radiobutton6,...
    handles.radiobutton7,handles.radiobutton8];
set(on,'Value',1)
set(handles.radiobutton2,'Value',0)  %gegenseitige Verriegelung
handles.FUP(2)=0;
guidata(handles.figure1, handles);   %speichern
% --------------------------------------------------------------------
% Callback for radiobutton2 - Prfschrittauswahl (Select a check )
% --------------------------------------------------------------------
function varargout = radiobutton2_Callback(h, eventdata, handles, varargin)
set([handles.radiobutton2,handles.radiobutton5],'Value',1)  %Verriegelung
off=[handles.radiobutton1,handles.radiobutton6,...
     handles.radiobutton7,handles.radiobutton8];
set(off,'Value',0)
handles.FUP(2)=1;
guidata(handles.figure1, handles);   %speichern
% --------------------------------------------------------------------
% Callback for radiobutton3 - Mit Ausgabekommentaren (All comments)
% --------------------------------------------------------------------
function varargout = radiobutton3_Callback(h, eventdata, handles, varargin)
set(handles.radiobutton3,'Value',1)  %gegenseitige Verriegelung
set(handles.radiobutton4,'Value',0)
handles.FUP(3)=1;
guidata(handles.figure1, handles);   %speichern
% --------------------------------------------------------------------
% Callback for radiobutton4 - Ohne Ausgabekommentare (No comments)
% --------------------------------------------------------------------
function varargout = radiobutton4_Callback(h, eventdata, handles, varargin)
set(handles.radiobutton3,'Value',0)  %gegenseitige Verriegelung
set(handles.radiobutton4,'Value',1)
handles.FUP(3)=0;
guidata(handles.figure1, handles);   %speichern
% --------------------------------------------------------------------
% Callback for radiobutton5 - Parameterprfung (Parameters)
% --------------------------------------------------------------------
function varargout = radiobutton5_Callback(h, eventdata, handles, varargin)
set([handles.radiobutton2,handles.radiobutton5],'Value',1)  %Verriegelung
off=[handles.radiobutton1,handles.radiobutton6,...
     handles.radiobutton7,handles.radiobutton8];
set(off,'Value',0)
handles.FUP(2)=1;
guidata(handles.figure1, handles);   %speichern
% --------------------------------------------------------------------
% Callback for radiobutton6 - Syntaxprfung (Syntax)
% --------------------------------------------------------------------
function varargout = radiobutton6_Callback(h, eventdata, handles, varargin)
set([handles.radiobutton2,handles.radiobutton6],'Value',1)  %Verriegelung
off=[handles.radiobutton1,handles.radiobutton5,...
     handles.radiobutton7,handles.radiobutton8];
set(off,'Value',0)
handles.FUP(2)=2;
guidata(handles.figure1, handles);   %speichern
% --------------------------------------------------------------------
% Callback for radiobutton7 - Sackgassenprfung (Dead ends)
% --------------------------------------------------------------------
function varargout = radiobutton7_Callback(h, eventdata, handles, varargin)
set([handles.radiobutton2,handles.radiobutton7],'Value',1)  %Verriegelung
off=[handles.radiobutton1,handles.radiobutton5,...
     handles.radiobutton6,handles.radiobutton8];
set(off,'Value',0)
handles.FUP(2)=3;
guidata(handles.figure1, handles);   %speichern
% --------------------------------------------------------------------
% Callback for radiobutton8 - Weitere Prfungen (Loops)
% --------------------------------------------------------------------
function varargout = radiobutton8_Callback(h, eventdata, handles, varargin)
set([handles.radiobutton2,handles.radiobutton8],'Value',1)  %Verriegelung
off=[handles.radiobutton1,handles.radiobutton5,...
     handles.radiobutton6,handles.radiobutton7];
set(off,'Value',0)
handles.FUP(2)=4;
guidata(handles.figure1, handles);   %speichern
