function [M,NAZ,NAF,ZOV,FEH,ANZ]=RELIZU03(M,NAZ,NAF,ZOV,ASCH)
%function [M,NAZ,NAF,ZOV,FEH,ANZ]=RELIZU03(M,NAZ,NAF,ZOV,ASCH)
%REDUZIERUNG DER TESTSCHRITTE DURCH ELIMINIERUNG LINEARER ZUSTANDSBERGNGE
%Unterschied zu RELIZU01: mit bergabe von NAF / ohne Ausgabe von Dim, SSE, SZE
%M    symmetrische Matrix die Zustandsverknpfungen in einem 
%     syntaxgeprften Funktionsplan beschreibt
%NAZ  Nummer des Anfangszustands, d.h. Spalten- bzw. Zeilennummer in Matrix M
%NAF  Nummer des Fehlerzustands, d.h. Spalten- bzw. Zeilennummer in Matrix M
%ZOV  Zuordnungsvektor fr reale Zustandsnummern zu Spalten/Zeilennr. in Matrix
%FEH=Cell-String mit Fehlermeldung, wenn ASCH(2)==1 
%ANZ=Cell-String-Vektor mit Ausgabetexten, wenn ASCH(1)==1 & ASCH(2)==1
%ASCH Textausgabeschalter ASCH(1)==ASCH: 0-nur Fehler; 1-alle Texte ausgeben
%                         ASCH(2)==GUI:  1-Textumleitung von Bildschirm in GUI

%  Copyright (c) 1997-2013. All Rights Reserved. 
%                     Dr. Gert-Helge Geitner; TU Dresden, Fak. EuI,
%                     Elektrotechnisches Institut (ETI); Mommsenstr. 13;
%                     D-01062 Dresden, Germany;
%                     http://eeiwzg.et.tu-dresden.de/ae2_files/ae_7_6e.htm

T_01='Abort: unsymmetric matrix passed!';
T_02='Abort: check not useful!';
T_03='Wrong initial state number ZA passed';
T_04='Wrong error state number ZF passed';
T_05='Number of eliminated state transitions via reduction : ';
T_06='Real initial state number via reduction: ';
T_07='Real error state number via reduction: ';

FEH={}; ANZ={};             %falls keine Rckmeldung erfolgt
if nargin==4                %Ausgabe alle Rckmeldetexte auf
   ASCH=1; GUI=0;           %Bildschirm freigeben
else GUI=ASCH(2);
     ASCH=ASCH(1); end
[Dim,i]=size(M);
if Dim~=i if GUI FEH={['RELIZU: ' T_01]}; return; else error(T_01); end; end
if Dim==2 if GUI FEH={['RELIZU: ' T_02]}; return; else error(T_02); end; end
[j,i]=size(NAZ);
if j~=1|i~=1|i>Dim if GUI FEH={['RELIZU: ' T_03]}; return;
                   else error(T_03); end; end
[j,i]=size(NAF);
if j~=1|i~=1|i>Dim if GUI FEH={['RELIZU: ' T_04]}; return;
                   else error(T_04); end; end

if NAF==0 i=0; else i=ZOV(NAF); end %Falls kein ZF vorgegeben
HSP=[ZOV(NAZ) i];  % Hilfsspeicher fr NAZ und NAF
SSE=sum(M);        % Summe der Spaltenelemente
SZE=sum(M');       % Summe der Zeilenelemente
VG=ones(1,Dim);    % Vergleichsvektor
SF=SSE==VG;        % Spaltenfilter auf einzelnes ODER
ZF=SZE==VG;        % Zeilenfilter auf einzelnes ODER

ii=0;                          % Eliminationszhler
i=1;                           % Nummer der zu untersuchenden Spalte
while i<=Dim                   % alle Spalten als Ausgangszustand betrachten
      if SF(i)==1              % Spaltenfilter erkennt einzelnes ODER 
         j=find(M(:,i)==1);    % Zeilenindex fr das einzelne ODER suchen
         if ZF(j)==1           % Eliminierbarer linearer Zustandswechsel?
            k=find(M(i,:)~=0); % Welche Zustnde schalten wie auf den zu ...
            for x=1:length(k)  % eliminierenden Zustand?
                M(j,k(x))=M(i,k(x)); % Umspeichern entspricht:
            end                      % Schalten auf linearen Nachfolgezustand
            if i==NAZ          % Falls Anfangszustand eliminiert: neu definieren
                         if i<j NAZ=j-1; else NAZ=j; end
            elseif i<NAZ NAZ=NAZ-1; % Falls Anfangszustand verschoben wird
            end
            if i==NAF          % Falls Fehlerzustand eliminiert: neu definieren
                         if i<j NAF=j-1; else NAF=j; end
            elseif i<NAF NAF=NAF-1; % Falls Fehlerzustand verschoben wird
            end
            M(:,i)=[];         % Zustandseliminierung: Streichen iter Spalte
            M(i,:)=[];         % und iter Zeile
            ZOV(i)=[];         % Streichen des iten realen Zustands
            i=i-1;             % Spaltennummer erneut verwenden
            ii=ii+1;           % Anzahl der eliminierten Zustnde
            Dim=Dim-1;         % Dimension korrigieren und neue Anfangswerte:
            SSE=sum(M);        % Summe der Spaltenelemente
            SZE=sum(M');       % Summe der Zeilenelemente
            VG=ones(1,Dim);    % Vergleichsvektor
            SF=SSE==VG;        % Spaltenfilter auf einzelnes ODER 
            ZF=SZE==VG;        % Zeilenfilter auf einzelnes ODER
            if Dim==2          % Minimum - weitere Reduzierung nicht sinnvoll
               i=2;            % while-Schleifen Abbruch
            end
         end
      end
      i=i+1;                   % nchste Spalte untersuchen
end
if ASCH~=0
   if GUI ANZ={[T_05 int2str(ii)]}; %Erste + hier unbedingte Ausgabe des Files
   else   disp([T_05 int2str(ii)]); end %Anzahl eliminierter Zustnde
   if NAF==0 i=0; else i=ZOV(NAF); end  %Falls kein ZF vorgegeben
   if HSP(1)~=ZOV(NAZ)
      if GUI ANZ(2,1)={[T_06 int2str(ZOV(NAZ))]};%Mit Sicherheit 2. Cell-Element
      else   disp([T_06 int2str(ZOV(NAZ))]);   %Aktuelle Nummer des Anfangszustands
      end
   end
   if HSP(2)~=i
      if GUI ANZ(end+1,1)={[T_07 int2str(i)]};%2. oder 3. Cell-Element
      else disp([T_07 int2str(i)]);           %Aktuelle Nummer des Fehlerzustands
      end
  end   
end

