function [BBZ,NUZ,SGZ,FEH,ANZ]=TAZUSA06(M,NAZ,NAF,ZOV,ASCH)
%Funktion [BBZ,NUZ,SGZ,FEH,ANZ]=TAZUSA06(M,NAZ,NAF,ZOV,ASCH)
%M=symmetrische Matrix als Ergebnis von XX2M und RELIZU
%NAZ=Nummer des Anfangszustands ZA
%NAF=Nummer des Fehlerzustands ZF
%ZOV=Zuordnungsvektor fr reale Zustandsnummern zu Spaltennummern in Matrix M
%ASCH=Textausgabeschalter und Abbruchschalter
%     ASCH(1)==ASCH2: 0-nur Fehler u. Ergebnisse; 1-alle Texte ausgeben
%     ASCH(2)==GUI:   1-Textumleitung von Bildschirm in GUI
%     ASCH(3)==ASCH:  1-Abbruch bei erster Sackgasse; bel.-vollstndige Prfung
%BBZ=bereits bearbeitete Zustnde
%NUZ=nicht untersuchte Zustnde
%SGZ=Sackgassenzustnde
%FEH=Cell-String mit Fehlermeldung, wenn ASCH(2)==1 
%ANZ=Cell-String-Vektor mit Ausgabetexten, wenn ASCH(1)==1 & ASCH(2)==1

%  Copyright (c) 1997-2013. All Rights Reserved. 
%                     Dr. Gert-Helge Geitner; TU Dresden, Fak. EuI,
%                     Elektrotechnisches Institut (ETI); Mommsenstr. 13;
%                     D-01062 Dresden, Germany;
%                     http://eeiwzg.et.tu-dresden.de/ae2_files/ae_7_6e.htm

T_01='Abort - starting this Function Chart impossibly!';
T_02='Initial state = ';
T_03='Abort at first detected dead end of state combinations = ';
T_04='No dead ends of state combinations detected.';
T_05='Number of dead ends for this Function Chart: ';
T_06='Number of analysed states: ';
T_07='State combinations=';

FEH={}; ANZ={};             %AW falls keine Rckmeldung erfolgt
if nargin==4                %Ausgabe aller Rckmeldetexte auf
   ASCH2=1; GUI=0; ASCH=1;  %Bildschirm freigeben, nur erste Sackgasse finden
else ASCH2=ASCH(1); GUI=ASCH(2); ASCH=ASCH(3); end

%Auswertung von M zur Aufstellung von MZF und MNZ
%MZF=Matrix Zustandsfilter, bezogen auf eine Weiterschalt-Spalte
%MNZ=Matrix neuer Zustand

[Zdim,Sdim]=size(M);
SM=sum(M); j=0;  PxZ=[]; PxZ_L=0;                %Vorbereitung Fallauswertung
MZF(Zdim,Zdim)=0; MNZ(Zdim,Zdim)=0;              %Dimension einstellen
for i=1:Sdim
   if     SM(i)>1   ii=find(M(:,i)==1);          %Fall 3: AxV; wre bei Erweiterung
                    for jj=1:SM(i) j=j+1; MZF(i,j)=1;%Vervielfachung betreffender
                        MNZ(ii(jj),j)=1;             %Spalten der AxV
                    end
   elseif SM(i)==-1 ii=find(M(:,i)==-1);         %Fall 4: PxZ; wre bei Erweiterung 
                    if isempty(find(PxZ(:)==ii)) %ZielZeile --> QuellSpalte und
                       PxZ_L=PxZ_L+1; PxZ(PxZ_L)=ii;%weitere Quellspalten streichen
                       j=j+1;
                       HV=M(ii,:); MZF(:,j)=HV'; %MZF(:,j)=M(ii,:) nicht mglich
                       MNZ(:,j)=M(:,i);
                    end
   else   j=j+1; MZF(i,j)=1; MNZ(:,j)=M(:,i);    %Fall 1/2: AxZ/PxV/EX; entspricht
   end                                           %ohne Flle 3/4 Diagonalmatrix
end
MZF=abs(MZF); MNZ=abs(MNZ);                      %fakultativ!?

[Zdim,Sdim]=size(MZF); VA=zeros(Zdim,1);%Vorbereitung einer Folgezustandsberechnung
VA(NAZ)=1;                       %Spaltenvektor Ausgangs- bzw. Untersuchungszustand    
WSCH=1; SU_MZF=sum(MZF); SGZ=[];        %Vorbereitung der while Schleife
if NAF==0 NUZ=[];
else      NUZ=zeros(Zdim,1); NUZ(NAF)=1;
end

%BEGINN erster Durchlauf: zur Vermeidung von Anlauffehlern ausprogrammieren
MAZ=[]; for i=1:Sdim MAZ=[MAZ,VA]; end %Punkt a: MAZ=aktueller Zustand=VA Sdim-fach
MGZ=MAZ&MZF;                           %Punkt b: MGZ=gefilterter Zustand
SD=~(SU_MZF-sum(MGZ));                 %Punkt c: Zeilenvektor SpaltenSummendiffrnzn
MMZ=[]; for i=1:Zdim MMZ=[MMZ; SD]; end%Punkt d: MMZ=Maske Ziel=SD Zdim-fach
ERG=(MAZ|(MNZ&MMZ))&(~MZF|~MMZ);       %Punkt e: neu
ii=find(sum(xor(MAZ,ERG)));            %Spaltennummern mglicher Folgezustnde
AnzFZ=length(ii);                      %Anzahl mglicher Folgezustnde
if AnzFZ==0                            %berhaupt keine Weiterschalten mglich?
   ii=find(VA);                        %Welche Zustnde sind aktiv?                         
   if GUI FEH={['TAZUSA: ' T_01]};     %Cell-Array
          FEH(2,1)={['      ' T_02 '[' int2str(ZOV(ii(:))) ']']}; return; %2. Zeile
   else 
    error([T_01 char(10) T_02 '[' int2str(ZOV(ii(:))) ']']);%BN's der Anfangssackga
   end
else
   BBZ=VA;                          %bearbeiteten Zustand in BBZ aufnehmen
   if AnzFZ==1&NAF==0               %Vermeidung von NUZ=[] --->NUZ=Anfangszustand
      NUZ=[NUZ,VA];                 %Doppelprfung unkritisch, nur ein Folgezustand
   end
   VA(:,1)=ERG(:,ii(1));            %neuen Ausgangszustand aus ERG holen
   for j=2:AnzFZ
      NUZ=[NUZ,ERG(:,ii(j))];       %weitere Folgezustnde in NUZ aufnehmen!
   end
end
%ENDE 1. Durchlauf zur Vermeidung: isempty(find(ismember([],[nicht leer],'rows')))

while WSCH==1 
%BEGINN Berechnung mglicher Folgezustnde ausgehend von e i n e m Zustand
MAZ=[]; for i=1:Sdim MAZ=[MAZ,VA]; end %Punkt a: MAZ=aktueller Zustand=VA Sdim-fach
MGZ=MAZ&MZF;                           %Punkt b: MGZ=gefilterter Zustand
SD=~(SU_MZF-sum(MGZ));                 %Punkt c: Zeilenvektor SpaltenSummendiffrnzn
MMZ=[]; for i=1:Zdim MMZ=[MMZ; SD]; end%Punkt d: MMZ=Maske Ziel=SD Zdim-fach
ERG=(MAZ|(MNZ&MMZ))&(~MZF|~MMZ);       %Punkt e: neue Zustnde bedingt setzen
                                       %         schaltbare aktive alte lschen
%ENDE Berechnung mglicher Folgezustnde
ii=find(sum(xor(MAZ,ERG)));            %Spaltennummern mglicher Folgezustnde
AnzFZ=length(ii);                      %Anzahl mglicher Folgezustnde
   if AnzFZ==0                         %berhaupt keine Weiterschalten mglich?
      BBZ=[BBZ,VA];                    %bearbeiteten Zustand in BBZ aufnehmen
      if VA==0                         %Nullvektor ausschlieen (Schleife!)
         if isempty(NUZ)
            WSCH=0;                    %keine weiteren nicht untersuchten Zustnde
         else
            VA(:,1)=NUZ(:,1);          %neuen Ausgangszustand aus NUZ holen
            NUZ(:,1)=[];               %diesen Zustand aus der NUZ-Menge streichen
         end          
         else                          %Normalfall
         ii=find(VA);                  %Welche Zustnde sind aktiv?
         VA(ii(:))=ZOV(ii(:));         %fr diese reale Zustandsnummern laden
         if ~isempty(SGZ)              %Anfangsladen?
            if isempty(find(ismember(SGZ',VA','rows'))) %noch nicht in SGZ?
               SGZ=[SGZ,VA];            %Zustand in Sackgassensammlung aufnehmen
            end
         else SGZ=[SGZ,VA];             %Zustand in Sackgassensammlung aufnehmen
         end
         if ASCH==1                     %Abbruch bei erster erkannter Sackgasse?
            HV=setdiff(ZOV(ii(:)),[])'; %"setdiff" wird nur zum sortieren verwendet
            if   GUI ANZ={[T_03]};      %mgliche 1. Ausgabe in Cell-Array
                     ANZ(2,1)={['      ' '[' int2str(HV) ']']}; %2. Zeile
            else     disp([T_03 '[' int2str(HV) ']']);
            end
            WSCH=0;                     %Zustandsnummern der ersten Sackgasse
         else                           %Ermittlung smtlicher Sackgassen!
            if isempty(NUZ)
               WSCH=0;                  %keine weiteren nicht untersuchten Zustnde
            else
               VA(:,1)=NUZ(:,1);        %neuen Ausgangszustand aus NUZ holen
               NUZ(:,1)=[];             %diesen Zustand aus der NUZ-Menge streichen
            end
         end
      end
   else                                 %es existieren Folgezustnde
      i=1;                              %ersten Folgezustand adressieren
      while i<=AnzFZ&~isempty(find(ismember(BBZ',ERG(:,ii(i))','rows')))
         i=i+1;
      end
      BBZ=[BBZ,VA];                     %bearbeiteten Zustand in BBZ aufnehmen
      if i>AnzFZ                        %keine neue Folgezustnde in ERG gefunden
         if isempty(NUZ)
            WSCH=0;                     %keine weiteren nicht untersuchten Zustnde
         else
            VA(:,1)=NUZ(:,1);           %neuen Ausgangszustand aus NUZ holen
            NUZ(:,1)=[];                %diesen Zustand aus der NUZ-Menge streichen
         end
      else                              %neue Folgezustnde in ERG enthalten
         VA(:,1)=ERG(:,ii(i));          %neuen Ausgangszustand aus ERG holen
         j=(find(ismember(NUZ',ERG(:,ii(i))','rows'))); %steht VA auch in NUZ?
         if ~isempty(j)
            NUZ(:,j)=[];                %diesen Zustand aus der NUZ-Menge streichen
         end
         for j=i+1:AnzFZ                %i adressiert schon verwendete Spalte
            if isempty(find(ismember(BBZ',ERG(:,ii(j))','rows'))) %nicht in BBZ?
               if isempty(find(ismember(NUZ',ERG(:,ii(j))','rows'))) %nicht in NUZ?
                  NUZ=[NUZ,ERG(:,ii(j))]; %in NUZ aufnehmen!
               end
            end
         end
      end
   end
   %[length(BBZ) length(NUZ) length(SGZ)]
end
[j,jj]=size(SGZ);                 %Anzahl der Sackgassen
if jj==0 
   if GUI ANZ={T_04};            %mgliche 1. Ausgabe dieses Files
   else   disp(T_04);            %keine Sackgassen gefunden
   end
elseif ASCH~=1
   if GUI ANZ={[T_05 int2str(jj)]};%mgliche 1. Ausgabe dieses Files
   else   disp([T_05 int2str(jj)]);
   end
   ERG=setxor(SGZ(:,1),0)';       %Hochkomma ab R12 notwendig
   for i=2:jj                     %Sortieren / einheitliche Lnge
      HV=setxor(SGZ(:,i),0)';     %Hochkomma ab R12 notwendig
      [m,n]=size(ERG);
      j=length(HV);
      if j<n
         HV(1,n-j+1:n)=HV(1,:);
         HV(1,1:n-j)=0;
      else
         ERG(:,j-n+1:j)=ERG(:,:);
         ERG(:,1:j-n)=0;
      end
      ERG=[ERG;HV];               %Anzeige untereinander
   end
   if GUI ANZ(2,1)={T_07};
          ANZ(3,1)={num2str(ERG)};
   else   Zustandskombinationen=ERG,     %','=direkte Bildschirmanzeige
   end
end
[j,jj]=size(BBZ);
if ASCH2
   if GUI ANZ(end+1,1)={[T_06 int2str(jj)]};%variabel anhngen
   else   disp([T_06 int2str(jj)]);         %Anzahl untersuchter Zustnde
   end
end
