function [MPF,EPF,MFP,EFP,FEH,ANZ]=TEDERU02(M,NAZ,NAF,ZOV,SKO,ASCH)
%Funktion [MPF,EPF,MFP,EFP,FEH,ANZ]=TEDERU02(M,NAZ,NAF,ZOV,SKO,ASCH)
%Test der Rckfhrschleifen auf Vorhandensein eines ZA- oder ZE-Blocks
%M    Symmetrische Matrix der Zustandsverknpfungen in einem FUP
%     Verknpfung senkrecht: 0=keine; +1=ODER; -1=UND
%NAZ  Nummer des Anfangszustands, d.h. Spalten- bzw. Zeilennr. in Matrix M
%NAF  Nummer des Fehlerzustands, d.h. Spalten- bzw. Zeilennr. in Matrix M
%ZOV  Zuordnungsvektor fr reale Zustandsnunmmern zu Spalten-/Zeilennr. in Matrix
%SKO  Vektor mit Zuordnung des SKodes zu ZOV
%ASCH=Textausgabeschalter ASCH(1)==ASCH: 0-nur Fehler; 1-alle Texte ausgeben
%                         ASCH(2)==GUI:  1-Textumleitung von Bildschirm in GUI
%MPF=Matrix aller Schleifen-Pfade ausgehend von ZA, Pfade zeilenweise
%MFP=Matrix aller Schleifen-Pfade ausgehend von ZF, Pfade zeilenweise
%EPF/EFP=Vektor mit Anzahl der ZE-/ZA-Blcke pro Pfad
%FEH=Cell-String mit Fehlermeldung bei ASCH(2)==1 
%ANZ=Cell-String-Vektor mit Ausgabetexten bei ASCH(2)==1; mgl. Verknpf.: ASCH(1)

%  Copyright (c) 1997-2013. All Rights Reserved. 
%                     Dr. Gert-Helge Geitner; TU Dresden, Fak. EuI,
%                     Elektrotechnisches Institut (ETI); Mommsenstr. 13;
%                     D-01062 Dresden, Germany;
%                     http://eeiwzg.et.tu-dresden.de/ae2_files/ae_7_6e.htm

%Voraussetzung ist ein auf Syntax geprfter FUP mit einer Funktionsplannummer FN:
% - es existiert genau ein Block ZA
% - es existiert ein oder kein Block ZF
% - es existieren keine doppelten Blocknummern BN

%MPO=Ordnung(Spalten-) von MPF bzw. MFP
%VPF=Vektor mit aktuellem Pfad
%SPF=Speichermatrix noch nicht fertig untersuchter Pfade
%SPZ=Zeilen  von SPF
%SPS=Spalten von SPF

ZA=4715; ZE=5027;

T_01='Abort ZA: several path break conditions fulfiled - error in M-matrix!';
T_02='Abort ZF: several path break conditions fulfiled - error in M-matrix!';
T_03='Abort: no block ZF, but unused state numbers!';
T_04='Abort: block ZF existent, but not all state numbers screened!';
T_05a='Error: following';
T_05b='Paths containing no block ZE';
T_05c='Warning: following';
T_05d='Paths containing more then one block ZE';
T_06='Abort: inconsistency in feedback path check for normal sequence!';
T_07='Abort: inconsistency in feedback pfad check for error handling!';
T_08='All feedback loops contain just one block ZA or ZE!';

FEH={}; ANZ={};             %falls keine Rckmeldung erfolgt
if nargin==5                %Ausgabe alle Rckmeldetexte auf
   ASCH=1; GUI=0;           %Bildschirm freigeben
else GUI=ASCH(2); ASCH=ASCH(1); end

%TEIL 1a Pfadermittlung ausgehend vom Anfangszustand
WSCH1=1; MPF=0; MPO=1; SPF(2,1)=NAZ; SPZ=2; SPS=1; %Vorbereitung while-Schleife
while WSCH1
   %Beginn Berechnung mglicher Pfade ausgehend von ZA
   VPF=SPF(SPZ,:); SPF(SPZ,:)=[]; SPZ=SPZ-1; %nicht fertig untersuchten Pfad laden
   VPF=VPF(find(VPF>0));                     %Nullen abspalten - Reihenfolge=Pfad 
   WSCH2=1;
   while WSCH2
      AV=M(:,VPF(end));                %Weiterschaltzustnde finden
      AV=find(AV~=0);                  %Nullen abspalten - Reihenfolge unwichtig
      LA=length(AV); LV=length(VPF)+1; %Vektorlngen bestimmen
      if LA>1                          %weitere mgliche Pfade speichern
         x=VPF;                        %Zwischenspeicher
         if LV>SPS
            SPF(1,LV)=0; SPS=LV;       %Matrixdimension vergrern
         else
            x(SPS)=0;                  %Vektordimension vergrern
         end
         for i=2:LA
             x(LV)=AV(i);              %alternative Pfade setzen
             SPF=[SPF;x];              %Speichermatrix laden
         end
         SPZ=SPZ+LA-1;                 %Zeilenanzahl korrigieren
      end
      %Prfen ob Zustandswiederholung vorliegt: ja --> Abbruch=Pfadende
      x=find(VPF==AV(1));              %Neuer Zustand schon im Pfad?
      if isempty(x)
         VPF(LV)=AV(1);                %Nein: Pfad mit zuf. 1. Verzw. erweitern
      else
         WSCH2=0;                      %Ja: innere while Schleife beenden
         if length(x)~=1
            if GUI FEH={['TEDERU: ' T_01]}; return;
            else   error(T_01); end; end %darf nur einmal vorkommen!
         VPF=VPF(x:end); LV=LV-x;      %nur Schleife + Zustand nur einmal:-1-(x-1)
         KoV=VPF; KoM=MPF;             %Arbeitskopien
         if     LV>MPO KoM(1,LV)=0; 
         elseif LV<MPO KoV(MPO)=0; end; %Lngen gleich
         KoM=sort(KoM,2); KoV=sort(KoV);%"ismember" arbeitet nur sortiert!
         if ~ismember(KoM,KoV,'rows')  %Pfad schon bekannt?
            if     LV>MPO MPO=LV; MPF(1,LV)=0;
            elseif LV<MPO         VPF(MPO)=0;  end
            MPF=[MPF; VPF];            %Pfad ist neu --> aufnehmen
         end   
      end
   end
   if SPZ==1 WSCH1=0; end              %uere while Schleife beenden
end
MPF(1,:)=[];                           %Leerzeile fr Anlauf streichen
[i,j]=size(MPF); ZOVLP=length(ZOV)+1; ZOV(ZOVLP)=0;
PNV=setdiff(1:ZOVLP-1,MPF(1:end));     %noch nicht verwendete Zustandsnummern
MPF=MPF+(xor(MPF,ones(i,j))*ZOVLP);    %so in Folgezeile ZOV(0) vermieden
MPF(:,:)=ZOV(MPF(:,:));               %Reelle Zustandsnummern laden

%TEIL 1b Pfadermittlung ausgehend vom Fehlerzustand
if NAF~=0
SPF=0; SPF(2,1)=NAF; ZFS=[];           %neue Vorbereitung der while-Schleife
WSCH1=1; MFP=0; MPO=1; SPZ=2; SPS=1;   %bekannte Vorbereitung
while WSCH1
   %Beginn Berechnung mglicher Pfade ausgehend von ZF
   VPF=SPF(SPZ,:); SPF(SPZ,:)=[]; SPZ=SPZ-1; %nicht fertig untersuchten Pfad laden
   VPF=VPF(find(VPF>0));                     %Nullen abspalten - Reihenfolge=Pfad 
   WSCH2=1;
   while WSCH2
      AV=M(:,VPF(end));                %Weiterschaltzustnde finden
      AV=find(AV~=0);                  %Nullen abspalten - Reihenfolge unwichtig
      LA=length(AV); LV=length(VPF)+1; %Vektorlngen bestimmen
      if LA>1                          %weitere mgliche Pfade speichern
         x=VPF;                        %Zwischenspeicher
         if LV>SPS
            SPF(1,LV)=0; SPS=LV;       %Matrixdimension vergrern
         else
            x(SPS)=0;                  %Vektordimension vergrern
         end
         for i=2:LA
             x(LV)=AV(i);              %alternative Pfade setzen
             SPF=[SPF;x];              %Speichermatrix laden
         end
         SPZ=SPZ+LA-1;                 %Zeilenanzahl korrigieren
      end
      %Prfen ob Zustandswiederholung vorliegt: ja --> Abbruch=Pfadende
      x=find(VPF==AV(1));              %Neuer Zustand schon im Pfad?
      if ~isempty(x)                   %Ja: innere while Schleife beenden
         WSCH2=0;                      
         if length(x)~=1
            if GUI FEH={['TEDERU: ' T_02]}; return;
            else   error(T_02); end; end %darf nur einmal vorkommen!
         VPF=VPF(x:end); LV=LV-x;      %nur Schleife + Zustand nur einmal:-1-(x-1)
         KoV=VPF; KoM=MFP;             %Arbeitskopien
         if     LV>MPO KoM(1,LV)=0; 
         elseif LV<MPO KoV(MPO)=0; end; %Lngen gleich
         KoM=sort(KoM,2); KoV=sort(KoV);%"ismember" arbeitet nur sortiert!
         if ~ismember(KoM,KoV,'rows')  %Pfad schon bekannt?
            if     LV>MPO MPO=LV; MFP(1,LV)=0;
            elseif LV<MPO         VPF(MPO)=0;  end
            MFP=[MFP; VPF];            %Pfad ist neu --> aufnehmen
         end
      else
         if ~isempty(intersect(PNV,AV(1)))
            VPF(LV)=AV(1);             %Nein: Pfad mit zuf. 1. Verzw. erweitern
         else
            WSCH2=0;                   %Ja: Pfad verlt hier Fehlerbehandlung
            ZFS=union(ZFS,VPF);        %Zustnde sammeln
         end
      end
   end
   if SPZ==1 WSCH1=0; end              %uere while Schleife beenden
end
MFP(1,:)=[];                           %Leerzeile fr Anlauf streichen
   if ~isempty(setxor(PNV,union(MFP(1:end),ZFS)))
      if GUI FEH={['TEDERU: ' T_04]}; return;
      else   error(FT_04); end; end
   if ~isempty(MFP)
      [i,j]=size(MFP);
      MFP=MFP+(xor(MFP,ones(i,j))*ZOVLP); %so in Folgezeile ZOV(0) vermieden
      MFP(:,:)=ZOV(MFP(:,:));             %Reelle Zustandsnummern laden
   else MFP=[]; end                       %Empty matrix: 0-by-0 herstellen
elseif ~isempty(PNV)
   if GUI FEH={['TEDERU: ' T_03]}; return;
   else   error(T_03); end                %noch unverwendete Zustnde brig
else MFP=[]; end                          %Kein Fehlerzustand programmiert

%TEIL 2 Pfaduntersuchung
x=find(SKO==ZA|SKO==ZE);                  %Positionen von ZA und ZE finden
x(:)=ZOV(x(:));                           %Positionen --> Blocknummern BN
EPF=sum(ismember(MPF,x),2); %Normalablauf:     nur einmal ZA oder ZE im Pfad?
if ~isempty(MFP)
EFP=sum(ismember(MFP,x),2); %Fehlerbehandlung: nur einmal ZA oder ZE im Pfad?
else EFP=[]; end

%if ASCH~=0
   sz=0;
   if any(EPF~=1)               %~=eye funktioniert nicht; obwohl ==eye OK
      j=find(EPF==0); k=find(EPF>1);
      if ~isempty(j)
         if GUI ANZ={[T_05a ' normal sequence - ' T_05b ' or ZA!']}; %1.Ausgabe
         else   disp([T_05a ' normal sequence - ' T_05b ' or ZA!']); end
         for i=1:length(j) E=find(MPF(j(i),:)~=0); 
                           sz=sz+1; s=num2str(sz);
             if GUI ANZ(end+1,1)={['path' s '=' num2str(MPF(j(i),1:E(end)))]};
             else            eval(['path' s '=MPF(j(i),1:E(end))']), end; end; end
      if ~isempty(k) 
         if GUI ANZ(end+1,1)={[T_05c ' normal sequence - ' T_05d ' or ZA!']};%vari.
         else            disp([T_05c ' normal sequence - ' T_05d ' or ZA!']); end
         for i=1:length(k) E=find(MPF(k(i),:)~=0); 
                           sz=sz+1; s=num2str(sz);
             if GUI ANZ(end+1,1)={['path' s '=' num2str(MPF(k(i),1:E(end)))]}; 
             else            eval(['path' s '=MPF(k(i),1:E(end))']), end; end; end
      if ~sz if GUI FEH={['TEDERU: ' T_06]}; return; else error(T_06); end; end
   end;
   szS=sz;
   if ~isempty(EFP)&any(EFP~=1) %~=eye funktioniert nicht; obwohl ==eye OK
      j=find(EFP==0); k=find(EFP>1);
      if ~isempty(j)       
         if GUI ANZ(end+1,1)={[T_05a ' error sequence - ' T_05b '!']}; %variabel
         else            disp([T_05a ' error sequence - ' T_05b '!']); end
         for i=1:length(j) E=find(MFP(j(i),:)~=0); 
                           sz=sz+1; s=num2str(sz);
             if GUI ANZ(end+1,1)={['path' s '=' num2str(MFP(j(i),1:E(end)))]};
             else            eval(['path' s '=MFP(j(i),1:E(end))']), end; end; end
      if ~isempty(k)      
         if GUI ANZ(end+1,1)={[T_05c ' error sequence - ' T_05d '!']};  %variabel
         else            disp([T_05c ' error sequence - ' T_05d '!']); end
         for i=1:length(k) E=find(MFP(k(i),:)~=0); 
                           sz=sz+1; s=num2str(sz);
             if GUI ANZ(end+1,1)={['path' s '=' num2str(MFP(k(i),1:E(end)))]};
             else            eval(['path' s '=MFP(k(i),1:E(end))']), end; end; end
      if sz==szS if GUI FEH={['TEDERU: ' T_07]}; return; else error(T_07); end; end
   end
   if ~sz if GUI ANZ={T_08};  %variabel nicht ntig
          else   disp(T_08);  end; end
%end
