function [M,NAZ,NAF,ZOV,SKO,FEH,ANZ]=XX2M04(XX,FN,ASCH)
%Function [M,NAZ,NAF,ZOV,SKO,FEH,ANZ]=XX2M04(XX,FN,ASCH)
%Generierung der symmetrischen Matrix M fr RELIZU
%XX=Matrix XX von FUPSCAN (Weiterschaltmatrix)
%FN=Funktionsplannummer aus XX
%ASCH=Textausgabeschalter ASCH(1)==ASCH: 0-nur Fehler; 1-alle Texte ausgeben
%                         ASCH(2)==GUI:  1-Textumleitung von Bildschirm in GUI
%M=Symmetrische Matrix der Zustandsverknpfungen in einem FUP
%  Verknpfung senkrecht: 0=keine; +1=ODER; -1=UND
%NAZ=Nummer des Anfangszustands, d.h. Spalten- bzw. Zeilennr. in Matrix
%NAF=Nummer des Fehlerzustands, d.h. Spalten- bzw. Zeilennr. in Matrix
%ZOV=Zuordnungsvektor fr reale Zustandsnunmmern zu Spalten-/Zeilennr. in Matrix
%SKO=Vektor mit Zuordnung des SKodes zu ZOV
%FEH=Cell-String mit Fehlermeldung, wenn ASCH(2)==1 
%ANZ=Cell-String-Vektor mit Ausgabetexten, wenn ASCH(1)==1 & ASCH(2)==1

%  Copyright (c) 1997-2013. All Rights Reserved. 
%                     Dr. Gert-Helge Geitner; TU Dresden, Fak. EuI,
%                     Elektrotechnisches Institut (ETI); Mommsenstr. 13;
%                     D-01062 Dresden, Germany;
%                     http://eeiwzg.et.tu-dresden.de/ae2_files/ae_7_6e.htm

%verwendete TKodes
TKZU=10; TKEX=80;
%verwendete SKodes --> SKO
EX=4892; ZA=4715; ZF=5105;
A2V=4701; A3V=4779; A2Z=5017; A3Z=5095;
P2V=5856; P3V=5934; P2Z=6172; P3Z=6250;
T_01='Abort: Function Chart devoid of or contains several blocks ZA!';
T_02='Warning - Function Chart devoid of block ZF!';
T_03='Abort: Function Chart contains several blocks ZF!';
T_10='Abort_A: ZIT/SNr no identification!';
T_20='Abort_B: ZIT/SNr no identification!';
T_21='Abort_B: check the syntax (Z1-E-Z2: E no EX block)!';
T_22='Abort_B: check the syntax (Z1-E-Z2: Z2 no identification)!';
T_30='Abort_C: ZIT/SNr no identification!';
T_31='Abort_C1: check the syntax!!!';
T_32='Abort_C2: check the syntax!!!';
T_33='Abort_CA: SNr no identification!';
T_34='Abort_CP: SNr no identification!';
T_40='Abort_D: ZIT/SNr no identification!';
T_41='Abort_D1: check the syntax!!!';
T_42a='         Warning - syntax special case II: AxV-EX-AxZ was ';
T_42b='-times programmed!';
T_43='Abort_DA: SNr no identification!';

FEH={}; ANZ={};             %falls keine Rckmeldung erfolgt
if nargin==2                %Ausgabe alle Rckmeldetexte auf
   ASCH=1; GUI=0;           %Bildschirm freigeben
else GUI=ASCH(2);
     ASCH=ASCH(1); end
A_SF_II=0;                  %Ausnahme: SyntaxSonderfall II rcksetzen
Z_SF_II=0;                  %Zhler fr A_SF_II
x=find(XX(:,7)==FN);        %alle Zustnde mit ausgewhlter FN
AnZu=length(x);             %Anzahl der Zustnde von FN
ZOV=XX(x(:),6);
SKO=XX(x(:),3);             %SKode fr externe ZE-Prfung bereit stellen
M(AnZu,AnZu)=0;             %Matrixdefinition = min. Rechenzeit
NAZ=find(XX(x(:),3)==ZA);
NAF=find(XX(x(:),3)==ZF);
if length(NAZ)~=1
   if GUI FEH={['XX2M: ' T_01]}; return; else error(T_01); end; end
if isempty(NAF) NAF=0; if ASCH
                       if GUI ANZ={['XX2M: ' T_02]};
                       else   disp([char(09) T_02]); end; end
elseif length(NAF)>1 
   if GUI FEH={['XX2M: ' T_03]}; return; else error(T_03); end; end

for s=1:AnZu
   ii=find(XX(:,2)==XX(x(s),9)&XX(:,4)==XX(x(s),10)); %1.Weiterschalten
   if length(ii)~=1
      if GUI FEH={['XX2M: ' T_10]}; return; else error(T_10); end; end
   SKodeA=XX(ii,3);
   TKodeA=XX(ii,5);
   OffsetA=XX(ii,8)/2;
   for j=1:OffsetA                           %wegen Alternativverzweigung
      jj=find(XX(:,2)==XX(ii,7+2*j)&XX(:,4)==XX(ii,8+2*j)); %2.Weiterschalten
      if length(jj)~=1 
         if GUI FEH={['XX2M: ' T_20]}; return; else error(T_20); end; end
      SKodeB=XX(jj,3);
      TKodeB=XX(jj,5);
      if TKodeB==TKZU                        %Zustandsblock erkannt
         if SKodeA~=EX
            if GUI FEH={['XX2M: ' T_21]}; return; else error(T_21); end; end
         z=find(XX(x(:),1)==XX(jj,1)); %Nummer der Zustands-Zeile in x suchen
         if length(z)~=1
            if GUI FEH={['XX2M: ' T_22]}; return; else error(T_22); end; end
         M(z,s)=1;                           %Zustand-Ereignis-Zustand erkannt
      else
         OffsetB=XX(jj,8)/2;
         for k=1:OffsetB                     %wegen Parallelverzweigung
            kk=find(XX(:,2)==XX(jj,7+2*k)&XX(:,4)==XX(jj,8+2*k)); %3.Weiterschalten
            if length(kk)~=1                 %bis 07/01: fehlerhaft jj
               if GUI FEH={['XX2M: ' T_30]}; return; else error(T_30); end; end
            SKodeC=XX(kk,3);
            TKodeC=XX(kk,5);
            if TKodeC~=TKZU            %[TKodeA SKodeA TKodeB SKodeB TKodeC SKodeC]
               if (SKodeA==A2V|SKodeA==A3V)&SKodeB==EX&(SKodeC==A2Z|SKodeC==A3Z)
                  mm=find(XX(:,2)==XX(kk,7+2*k)&XX(:,4)==XX(kk,8+2*k));%4.Weitersch
                  if length(mm)~=1 
                     if GUI FEH={['XX2M: ' T_40]}; return; 
                     else error(T_40); end; end
                  TKodeD=XX(mm,5);
                  if TKodeD~=TKZU
                     if GUI FEH={['XX2M: ' T_41]}; return; else error(T_41); end;
                  else             %SyntaxSonderfall II: Zustand-AxV-EX-AxZ-Zustand
                     z=find(XX(x(:),1)==XX(mm,1));%Nummer Zustnds-Zeile in x suchen
                     if length(z)~=1
                        if GUI FEH={['XX2M: ' T_43]}; return;
                        else error(T_43); end;end
                     M(z,s)=1;                 %Zustand-{E/AZ,AV/E}-Zustand erkannt
                                               %Merker u. Zhler Ausnahme SF II
                     A_SF_II=1; Z_SF_II=Z_SF_II+1; end
               else
                  if GUI FEH={['XX2M: ' T_31]}; return;
                  else error(T_31); end; end; end
            if ((SKodeA==A2V|SKodeA==A3V)&TKodeB==TKEX)|...
               ((SKodeB==A2Z|SKodeB==A3Z)&TKodeA==TKEX) 
               z=find(XX(x(:),1)==XX(kk,1)); %Nummer der Zustands-Zeile in x suchen
               if length(z)~=1
                  if ~A_SF_II
                     if GUI FEH={['XX2M: ' T_33]}; return; else error(T_33); end;
                  else  A_SF_II=0; end     %Ausnahme SyntaxSonderfall II rcksetzen
               else M(z,s)=1; end          %Zustand-{E/AZ,AV/E}-Zustand erkannt  
            elseif ((SKodeB==P2V|SKodeB==P3V)&TKodeA==TKEX)|...
                   ((SKodeA==P2Z|SKodeA==P3Z)&TKodeB==TKEX)
               z=find(XX(x(:),1)==XX(kk,1)); %Nummer der Zustands-Zeile in x suchen
               if length(z)~=1
                  if GUI FEH={['XX2M: ' T_34]}; return; else error(T_34); end; end
               M(z,s)=-1;                    %Zustand-{E/PV,PZ/E}-Zustand erkannt
            else
               if GUI FEH={['XX2M: ' T_32]}; return;
               else error(T_32); end         %darf nie erreicht werden!!
            end
         end
      end
   end
end
if ASCH if Z_SF_II text=[T_42a num2str(Z_SF_II) T_42b];
                   if GUI ANZ={text}; else disp(text); end; end; end
