function [M,NAZ,NAF,ZOV,FEH,ANZ]=ZUKOM03(M,NAZ,NAF,ZOV,ASCH)
%function [M,NAZ,NAF,ZOV,FEH,ANZ]=ZUKOM03(M,NAZ,NAF,ZOV,ASCH)
%REDUZIERUNG DER TESTSCHRITTE DURCH ZUSTANDSKOMPRIMIERUNG MITTELS
%ELIMINIERUNG VON ZUSTANDSBERGNGEN
%M    symmetrische Matrix die Zustandsverknpfungen in einem syntaxgeprften
%     Funktionsplan beschreibt
%NAZ  Nummer des Anfangszustands, d.h. Spalten- bzw. Zeilennummer in Matrix M
%NAF  Nummer des Fehlerzustands, d.h. Spalten- bzw. Zeilennummer in Matrix M
%ZOV  Zuordnungsvektor fr reale Zustandsnummern zu Spalten/Zeilennr. in Matrix
%ASCH=Textausgabeschalter ASCH(1)==ASCH: 0-nur Fehler; 1-alle Texte ausgeben
%                         ASCH(2)==GUI:  1-Textumleitung von Bildschirm in GUI
%FEH=Cell-String mit Fehlermeldung, wenn ASCH(2)==1 
%ANZ=Cell-String-Vektor mit Ausgabetexten, wenn ASCH(1)==1 & ASCH(2)==1

%  Copyright (c) 1997-2013. All Rights Reserved. 
%                     Dr. Gert-Helge Geitner; TU Dresden, Fak. EuI,
%                     Elektrotechnisches Institut (ETI); Mommsenstr. 13;
%                     D-01062 Dresden, Germany;
%                     http://eeiwzg.et.tu-dresden.de/ae2_files/ae_7_6e.htm

T_01='ZUKOM: Abort - unsymmetric matrix passed!';
T_02='State compression not useful (Dim<6)!';
T_03='ZUKOM: wrong initial state number ZA passed';
T_04='ZUKOM: wrong error state number ZF passed';
T_05='Sum of eliminated state blocks via compression : ';
T_06='Real initial state number via compression: ';
T_07='Real error state number via compression: ';
T_08a='Compression run ';
T_08b=' states eliminated';
T_22='ZUKOM: field(2,2) ambiguous!';
T_23='ZUKOM: field(2,3) ambiguous!';
T_24='ZUKOM: field(2,4) ambiguous!';
T_44='ZUKOM: field(4,4) ambiguous!';

FEH={}; ANZ={};             %falls keine Rckmeldung erfolgt
if nargin==4                %Ausgabe aller Rckmeldetexte auf
   ASCH=1; GUI=0;           %Bildschirm freigeben
else GUI=ASCH(2); ASCH=ASCH(1); end

[Dim,i]=size(M);
if Dim~=i if GUI FEH={T_01}; return; else error(T_01); end; end
if Dim<6 if GUI ANZ={T_02}; %erste Ausgabe dieses Files
         else   disp(T_02); end;
         return; end
[j,i]=size(NAZ);
if j~=1|i~=1|i>Dim if GUI FEH={T_03}; return; else error(T_03); end; end
[j,i]=size(NAF);
if j~=1|i~=1|i>Dim if GUI FEH={T_04}; return; else error(T_04); end; end

if NAF==0 i=0; else i=ZOV(NAF); end %Falls kein ZF vorgegeben
HSP=[ZOV(NAZ) i]; %Hilfsspeicher fr NAZ und NAF
SM=sum(M);        %Summe der Spaltenelemente
KOM=0;            %Anfangswert Kompressionsschalter

ii=0;             %Kompressionszhler
ii_SP=0;          %nur fr Ergebnisanzeige der einzelnen Lufe
i=1;              %Anfangswert Spaltennummer
BST=1;            %Berechnungssteuerung 1= Hauptdiagonale
                  %                     2= Feld{2,1;1,2}
                  %                     3= Feld(2,3)
                  %Fr Elemente von M gilt E=!{0,1,-1}
                  %BST={4,5} nicht ohne nderungen einsetzbar, da eine Ver-
                  %mischung der Elemente {1,-1} auf den Spalten eintreten kann!
                  %Mgliche Lsung: M-Speicher und Spaltenprfung nach Rechnung?
while BST<4 %BST dynamisch gendert durch bergabe mglich
ii_alt=ii;  %Wiederholung mit BST=konst. ermglichen
i=1;        %Spaltenauswahl
while i<=Dim          %alle Spalten als Ausgangszustand betrachten
   jj=find(M(:,i)~=0); %Zeilen=Weiterschaltzustnde
   if SM(i)>=1
      if SM(i)>1               %Quelle=AxV
         for j=1:length(jj)
            M_SP=M;            %Matrix-M-Speicher falls adr==NAZ|adr==NAF
            adr=jj(j);         %Laufzeit minimieren
            if SM(adr)>=1
               if SM(adr)>1    %Feld(1,1)=AxV/AxV
                  if BST==1
                     KOM=1;
                     M(:,i)=M(:,i)+M(:,adr); %1x Spalte: Mitte-->Oben
                  end
               else            %Feld(2,1)=AxV/AxZ
                  if BST==2
                     KOM=1;
                     M(:,i)=M(:,i)|M(:,adr); %1x Spalte: Mitte-->Oben
                  end                        %"|" --> "+" wre falsch fr gleiche
               end                           %AxZ-Ziele, da dann "2" resultiert
            else
               if SM(adr)<-1   %Feld(3,1)=AxV/PxV
                               %X
               else            %Feld(4,1)=AxV/PxZ
                               %X
               end
            end
            if KOM==1&adr~=NAZ&adr~=NAF %NAZ/NAF-nderung wre mehrdeutig
                                        %i, adr, ZOV', M, %IM TESTFALL!!!
               if adr<NAZ NAZ=NAZ-1; end %Falls Anfangszustand verschoben wird
               if adr<NAF NAF=NAF-1; end %Falls Fehlerzustand verschoben wird
               M(:,adr)=[];      %ZustandsKomprimierung: Streichen der Spalte
               M(adr,:)=[];      %und der Zeile mit Nummer="adr"
               ZOV(adr)=[];      %Streichen des realen Zustands: Nummer="adr"
               if adr<i i=i-1; end %Falls eine schon bearbeitete Spalte wegfllt
               for m=j+1:length(jj) %jj-Speicher Spaltennummern korrigieren
                  if jj(m)>adr jj(m)=jj(m)-1; end
               end
               ii=ii+1;          %Anzahl der eliminierten Zustnde
               Dim=Dim-1;        %Dimension korrigieren und neue Anfangswerte:
               SM=sum(M);        %Summe der Spaltenelemente
               if Dim<6          %Minimum - weitere Reduzierung nicht sinnvoll
                  Dim=0; break;  %Abbruch der innersten Schleife
               end
               KOM=0;            %Komprimierungsschalter rcksetzen
            elseif KOM==1&(adr==NAZ|adr==NAF)
               M=M_SP;           %schon vernderte Matrix Rckspeichern
               KOM=0;            %Komprimierungsschalter rcksetzen
            end
         end
      else                     %Quelle=AxZ
         for j=1:length(jj)
            M_SP=M;            %Matrix-M-Speicher falls adr==NAZ|adr==NAF
            adr=jj(j);         %Laufzeit minimieren
            if SM(adr)>=1
               if SM(adr)>1    %Feld(1,2)=AxZ/AxV
                  if BST==2
                     KOM=1;
                     FS=find(M(adr,:)==1); %FS-Feldspeicher
                     for k=1:length(FS)
                        M(:,FS(k))=M(:,FS(k))+M(:,adr); %Xx Spalte: Mitte-->Oben
                     end
                  end
               else            %Feld(2,2)=AxZ/AxZ
                  if BST==1
                     KOM=1;
                     FS=find(M(:,adr)==1); %FS-Feldspeicher
                     if length(FS)~=1 if GUI FEH={T_22}; return;
                                      else   error(T_22); end; end
                     M(FS,:)=M(FS,:)+M(adr,:); %1x Zeile: Mitte-->Unten
                  end
               end
            else
               if SM(adr)<-1   %Feld(3,2)=AxZ/PxV
                  if BST==5
                     KOM=1;
                     FS=find(M(adr,:)==1); %FS-Feldspeicher
                     for k=1:length(FS)
                        M(:,FS(k))=M(:,FS(k))+M(:,adr); %Xx Spalte: Mitte-->Oben
                     end
                  end
               else            %Feld(4,2)=AxZ/PxZ
                               %X
               end
            end
            if KOM==1&adr~=NAZ&adr~=NAF %NAZ/NAF-nderung wre mehrdeutig
                                        %i, adr, ZOV', M, %IM TESTFALL!!!
               if adr<NAZ NAZ=NAZ-1; end %Falls Anfangszustand verschoben wird
               if adr<NAF NAF=NAF-1; end %Falls Fehlerzustand verschoben wird
               M(:,adr)=[];      %ZustandsKomprimierung: Streichen der Spalte
               M(adr,:)=[];      %und der Zeile mit Nummer="adr"
               ZOV(adr)=[];      %Streichen des realen Zustands: Nummer="adr"
               if adr<i i=i-1; end %Falls eine schon bearbeitete Spalte wegfllt
               for m=j+1:length(jj) %jj-Speicher Spaltennummern korrigieren
                  if jj(m)>adr jj(m)=jj(m)-1; end
               end
               ii=ii+1;          %Anzahl der eliminierten Zustnde
               Dim=Dim-1;        %Dimension korrigieren und neue Anfangswerte:
               SM=sum(M);        %Summe der Spaltenelemente
               if Dim<6          %Minimum - weitere Reduzierung nicht sinnvoll
                  Dim=0; break;  %Abbruch der innersten Schleife
               end
               KOM=0;            %Komprimierungsschalter rcksetzen
            elseif KOM==1&(adr==NAZ|adr==NAF)
               M=M_SP;           %schon vernderte Matrix Rckspeichern
               KOM=0;            %Komprimierungsschalter rcksetzen
            end
         end
      end
   else
      if SM(i)<-1              %Quelle=PxV
         for j=1:length(jj)
            M_SP=M;            %Matrix-M-Speicher falls adr==NAZ|adr==NAF
            adr=jj(j);         %Laufzeit minimieren
            if SM(adr)>=1
               if SM(adr)>1    %Feld(1,3)=PxV/AxV
                               %X
               else            %Feld(2,3)=PxV/AxZ
                  if BST==3
                     KOM=1;
                     FS=find(M(:,adr)==1); %FS-Feldspeicher
                     if length(FS)~=1 if GUI FEH={T_23}; return;
                                      else   error(T_23); end; end
                     M(FS,:)=M(FS,:)+M(adr,:); %1x Zeile: Mitte-->Unten
                  end
               end
            else
               if SM(adr)<-1   %Feld(3,3)=PxV/PxV
                  if BST==1
                     KOM=1;
                     M(:,i)=M(:,i)+M(:,adr); %1x Spalte: Mitte-->Oben
                  end
               else            %Feld(4,3)=PxV/PxZ
                               %X
               end
            end
            if KOM==1&adr~=NAZ&adr~=NAF %NAZ/NAF-nderung wre mehrdeutig
                                        %i, adr, ZOV', M, %IM TESTFALL
               if adr<NAZ NAZ=NAZ-1; end %Falls Anfangszustand verschoben wird
               if adr<NAF NAF=NAF-1; end %Falls Fehlerzustand verschoben wird
               M(:,adr)=[];      %ZustandsKomprimierung: Streichen der Spalte
               M(adr,:)=[];      %und der Zeile mit Nummer="adr"
               ZOV(adr)=[];      %Streichen des realen Zustands: Nummer="adr"
               if adr<i i=i-1; end %Falls eine schon bearbeitete Spalte wegfllt
               for m=j+1:length(jj) %jj-Speicher Spaltennummern korrigieren
                  if jj(m)>adr jj(m)=jj(m)-1; end
               end
               ii=ii+1;          %Anzahl der eliminierten Zustnde
               Dim=Dim-1;        %Dimension korrigieren und neue Anfangswerte:
               SM=sum(M);        %Summe der Spaltenelemente
               if Dim<6          %Minimum - weitere Reduzierung nicht sinnvoll
                  Dim=0; break;  %Abbruch der innersten Schleife
               end
               KOM=0;            %Komprimierungsschalter rcksetzen
            elseif KOM==1&(adr==NAZ|adr==NAF)
               M=M_SP;           %schon vernderte Matrix Rckspeichern
               KOM=0;            %Komprimierungsschalter rcksetzen
            end
         end
      else                     %Quelle=PxZ
         for j=1:length(jj)
            M_SP=M;            %Matrix-M-Speicher falls adr==NAZ|adr==NAF
            adr=jj(j);         %Laufzeit minimieren
            if SM(adr)>=1
               if SM(adr)>1    %Feld(1,4)=PxZ/AxV
                               %X
               else            %Feld(2,4)=PxZ/AxZ
                  if BST==4
                     KOM=1;
                     FS=find(M(:,adr)==1); %FS-Feldspeicher
                     if length(FS)~=1 if GUI FEH={T_24}; return;
                                      else   error(T_24); end; end
                     M(FS,:)=M(FS,:)+M(adr,:); %1x Zeile: Mitte-->Unten
                  end
               end
            else
               if SM(adr)<-1   %Feld(3,4)=PxZ/PxV
                               %X
               else            %Feld(4,4)=PxZ/PxZ
                  if BST==1
                     KOM=1;
                     FS=find(M(:,adr)==1); %FS-Feldspeicher
                     if length(FS)~=1 if GUI FEH={T_44}; return;
                                      else   error(T_44); end; end %i,adr,ZOV',M,
                     M(FS,:)=M(FS,:)+M(adr,:); %1x Zeile: Mitte-->Unten
                  end
               end
            end
            if KOM==1&adr~=NAZ&adr~=NAF %NAZ/NAF-nderung wre mehrdeutig
                                        %i, adr, ZOV', M, %IM TESTFALL
               if adr<NAZ NAZ=NAZ-1; end %Falls Anfangszustand verschoben wird
               if adr<NAF NAF=NAF-1; end %Falls Fehlerzustand verschoben wird
               M(:,adr)=[];      %ZustandsKomprimierung: Streichen der Spalte
               M(adr,:)=[];      %und der Zeile mit Nummer="adr"
               ZOV(adr)=[];      %Streichen des realen Zustands: Nummer="adr"
               if adr<i i=i-1; end %Falls eine schon bearbeitete Spalte wegfllt
               for m=j+1:length(jj) %jj-Speicher Spaltennummern korrigieren
                  if jj(m)>adr jj(m)=jj(m)-1; end
               end
               ii=ii+1;          %Anzahl der eliminierten Zustnde
               Dim=Dim-1;        %Dimension korrigieren und neue Anfangswerte:
               SM=sum(M);        %Summe der Spaltenelemente
               if Dim<6          %Minimum - weitere Reduzierung nicht sinnvoll
                  Dim=0; break;  %Abbruch der innersten Schleife
               end
               KOM=0;            %Komprimierungsschalter rcksetzen
            elseif KOM==1&(adr==NAZ|adr==NAF)
               M=M_SP;           %schon vernderte Matrix Rckspeichern
               KOM=0;            %Komprimierungsschalter rcksetzen
            end
         end
      end
   end                           %ENDE Bearbeitung einer Spalte
   %if (BST==2&ii==3&ii~=ii_alt)|(BST==3&ii_alt~=ii) [ii i adr],ZOV',M, end %?????
   %M,ZOV',i, SM(i), Dim  
   i=i+1;                 %nchste Spalte untersuchen
end                       %ENDE innere WHILE-SCHLEIFE
if ii==ii_alt             %Wiederholung mit BST=const. ermglichen
   if GUI                 %variabel wegen Schleife ntig
         ANZ(end+1,1)={[T_08a int2str(BST) ': ' int2str(ii-ii_SP) T_08b]}; 
   else           disp([T_08a int2str(BST) ': ' int2str(ii-ii_SP) T_08b]); end
   ii_SP=ii;                 %nur fr Ergebnisanzeige der einzelnen Lufe
   BST=BST+1;                %Berechnungssteuerung=Auswahl der Bearbeitungsfelder
end
end                       %ENDE uere WHILE-SCHLEIFE
if ASCH~=0
   if GUI ANZ(end+1,1)={[T_05 int2str(ii)]};     %variabel ntig s.o.
   else            disp([T_05 int2str(ii)]); end %Anzahl eliminierter Zustnde
   if NAF==0 i=0; else i=ZOV(NAF); end %Falls kein ZF vorgegeben
   if HSP(1)~=ZOV(NAZ)                 %Aktuelle Nummer des Anfangszustands
      if GUI ANZ(end+1,1)={[T_06 int2str(ZOV(NAZ))]};  %variabel ntig s.o. 
      else            disp([T_06 int2str(ZOV(NAZ))]); end; end
   if HSP(2)~=i                        %Aktuelle Nummer des Fehlerzustands
      if GUI ANZ(end+1,1)={[T_07 int2str(i)]};     %variabel ntig s.o.
      else            disp([T_07 int2str(i)]); end; end
end
